@@ -85,8 +85,9 @@ Magick::Drawable& Magick::Drawable::operator= (const Magick::Drawable& original_
 {
   if (this != &original_)
     {
+      DrawableBase* temp_dp = (original_.dp ? original_.dp->copy() : 0);
       delete dp;
-      dp = (original_.dp ? original_.dp->copy() : 0);
+      dp = temp_dp;
     }
   return *this;
 }
@@ -164,8 +165,9 @@ Magick::VPath& Magick::VPath::operator= (const Magick::VPath& original_ )
 {
   if (this != &original_)
     {
+      VPathBase* temp_dp = (original_.dp ? original_.dp->copy() : 0);
       delete dp;
-      dp = (original_.dp ? original_.dp->copy() : 0);
+      dp = temp_dp;
     }
   return *this;
 }
@@ -263,7 +265,8 @@ Magick::DrawableBezier::DrawableBezier ( const CoordinateList &coordinates_ )
 }
 // Copy constructor
 Magick::DrawableBezier::DrawableBezier( const Magick::DrawableBezier& original_ )
-  : _coordinates(original_._coordinates)
+  : DrawableBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 // Destructor
@@ -319,7 +322,8 @@ Magick::DrawablePushClipPath::DrawablePushClipPath( const std::string &id_)
 }
 Magick::DrawablePushClipPath::DrawablePushClipPath
 ( const Magick::DrawablePushClipPath& original_ ) //multithread safe const char*
-  : _id(original_._id.c_str())
+  : DrawableBase (original_),
+    _id(original_._id.c_str())
 {
 }
 Magick::DrawablePushClipPath::~DrawablePushClipPath( void )
@@ -344,7 +348,8 @@ Magick::DrawableClipPath::DrawableClipPath( const std::string &id_ )
 }
 
 Magick::DrawableClipPath::DrawableClipPath ( const Magick::DrawableClipPath& original_ )
-  : _id(original_._id.c_str())
+  : DrawableBase (original_),
+    _id(original_._id.c_str())
 {
 }
 Magick::DrawableClipPath::~DrawableClipPath( void )
@@ -490,15 +495,17 @@ Magick::DrawableCompositeImage& Magick::DrawableCompositeImage::operator=
       _y = original_._y;
       _width = original_._width;
       _height = original_._height;
+      Image* temp_image = new Image(*original_._image);
       delete _image;
-      _image = new Image(*original_._image);
+      _image = temp_image;
     }
   return *this;
 }
 void Magick::DrawableCompositeImage::filename( const std::string &filename_ )
 {
+  Image* temp_image = new Image(filename_);
   delete _image;
-  _image = new Image(filename_);
+  _image = temp_image;
 }
 std::string Magick::DrawableCompositeImage::filename( void ) const
 {
@@ -507,8 +514,9 @@ std::string Magick::DrawableCompositeImage::filename( void ) const
 
 void Magick::DrawableCompositeImage::image( const Magick::Image &image_ )
 {
+  Image* temp_image = new Image(image_);
   delete _image;
-  _image = new Image(image_);
+  _image = temp_image;
 }
 Magick::Image Magick::DrawableCompositeImage::image( void ) const
 {
@@ -559,7 +567,8 @@ Magick::DrawableFillColor::DrawableFillColor( const Magick::Color &color_ )
 }
 Magick::DrawableFillColor::DrawableFillColor
 ( const Magick::DrawableFillColor& original_ )
-  : _color(original_._color)
+  : DrawableBase (original_),
+    _color(original_._color)
 {
 }
 Magick::DrawableFillColor::~DrawableFillColor( void )
@@ -625,7 +634,8 @@ Magick::DrawableFont::DrawableFont ( const std::string &family_,
 {
 }
 Magick::DrawableFont::DrawableFont ( const Magick::DrawableFont& original_ )
-  : _font(original_._font),
+  : DrawableBase (original_),
+    _font(original_._font),
     _family(original_._family),
     _style(original_._style),
     _weight(original_._weight),
@@ -709,7 +719,8 @@ Magick::DrawablePath::DrawablePath ( const VPathList &path_ )
 {
 }
 Magick::DrawablePath::DrawablePath ( const Magick::DrawablePath& original_ )
-  : _path(original_._path)
+  : DrawableBase (original_),
+    _path(original_._path)
 {
 }
 Magick::DrawablePath::~DrawablePath ( void )
@@ -764,7 +775,8 @@ Magick::DrawablePolygon::DrawablePolygon ( const CoordinateList &coordinates_ )
 }
 Magick::DrawablePolygon::DrawablePolygon
 ( const Magick::DrawablePolygon& original_ )
-  : _coordinates(original_._coordinates)
+  : DrawableBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::DrawablePolygon::~DrawablePolygon ( void )
@@ -803,7 +815,8 @@ Magick::DrawablePolyline::DrawablePolyline
 }
 Magick::DrawablePolyline::DrawablePolyline
 ( const Magick::DrawablePolyline& original_ )
-  : _coordinates(original_._coordinates)
+  : DrawableBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::DrawablePolyline::~DrawablePolyline ( void )
@@ -889,7 +902,8 @@ Magick::DrawablePushPattern::DrawablePushPattern
 }
 Magick::DrawablePushPattern::DrawablePushPattern
 ( const Magick::DrawablePushPattern& original_ )
-  : _id(original_._id),
+  : DrawableBase (original_),
+    _id(original_._id),
     _x(original_._x),
     _y(original_._y),
     _width(original_._width),
@@ -1011,7 +1025,8 @@ Magick::DrawableDashArray::DrawableDashArray( const unsigned int* dasharray_ )
 }
 Magick::DrawableDashArray::DrawableDashArray
 (const Magick::DrawableDashArray& original_)
-  : _size(0),
+  : DrawableBase (original_),
+    _size(0),
     _dasharray(0)
 {
   dasharray( original_._dasharray );
@@ -1042,7 +1057,7 @@ Magick::DrawableBase* Magick::DrawableDashArray::copy() const
 }
 void Magick::DrawableDashArray::dasharray ( const double* dasharray_ )
 {
-  LiberateMemory(reinterpret_cast<void**>(&_dasharray));
+  MagickFreeMemory(_dasharray);
 
   if(dasharray_)
     {
@@ -1056,7 +1071,7 @@ void Magick::DrawableDashArray::dasharray ( const double* dasharray_ )
       _size = n;
 
       // Allocate elements
-      _dasharray=static_cast<double*>(AcquireMemory((n+1)*sizeof(double)));
+      _dasharray=MagickAllocateMemory(double*,(n+1)*sizeof(double));
       // Copy elements
       {
         double *q = _dasharray;
@@ -1071,7 +1086,7 @@ void Magick::DrawableDashArray::dasharray ( const double* dasharray_ )
 // code to the const double* version.
 void Magick::DrawableDashArray::dasharray( const unsigned int* dasharray_ )
 {
-  LiberateMemory(reinterpret_cast<void**>(&_dasharray));
+  MagickFreeMemory(_dasharray);
 
   if(dasharray_)
     {
@@ -1085,7 +1100,7 @@ void Magick::DrawableDashArray::dasharray( const unsigned int* dasharray_ )
       _size = n;
 
       // Allocate elements
-      _dasharray=static_cast<double*>(AcquireMemory((n+1)*sizeof(double)));
+      _dasharray=MagickAllocateMemory(double*,(n+1)*sizeof(double));
       // Copy elements
       {
         double *q = _dasharray;
@@ -1175,7 +1190,8 @@ Magick::DrawableStrokeColor::DrawableStrokeColor
 }
 Magick::DrawableStrokeColor::DrawableStrokeColor
 ( const Magick::DrawableStrokeColor& original_ )
-  : _color(original_._color)
+  : DrawableBase (original_),
+    _color(original_._color)
 {
 }
 Magick::DrawableStrokeColor::~DrawableStrokeColor ( void )
@@ -1238,7 +1254,8 @@ Magick::DrawableText::DrawableText ( const double x_, const double y_,
 {
 }
 Magick::DrawableText::DrawableText( const Magick::DrawableText& original_ )
-  : _x(original_._x),
+  : DrawableBase (original_),
+    _x(original_._x),
     _y(original_._y),
     _text(original_._text),
     _encoding(original_._encoding)
@@ -1265,7 +1282,8 @@ Magick::DrawableTextAntialias::DrawableTextAntialias ( bool flag_ )
 {
 }
 Magick::DrawableTextAntialias::DrawableTextAntialias( const Magick::DrawableTextAntialias &original_ )
-  : _flag(original_._flag)
+  : DrawableBase (original_),
+    _flag(original_._flag)
 {
 }
 Magick::DrawableTextAntialias::~DrawableTextAntialias ( void )
@@ -1289,7 +1307,8 @@ Magick::DrawableTextDecoration::DrawableTextDecoration
 }
 Magick::DrawableTextDecoration::DrawableTextDecoration
   ( const Magick::DrawableTextDecoration &original_ )
-    : _decoration(original_._decoration)
+    : DrawableBase (original_),
+      _decoration(original_._decoration)
 {
 }
 Magick::DrawableTextDecoration::~DrawableTextDecoration( void )
@@ -1313,7 +1332,8 @@ Magick::DrawableTextUnderColor::DrawableTextUnderColor
 }
 Magick::DrawableTextUnderColor::DrawableTextUnderColor
 ( const Magick::DrawableTextUnderColor& original_ )
-  : _color(original_._color)
+  : DrawableBase (original_),
+    _color(original_._color)
 {
 }
 Magick::DrawableTextUnderColor::~DrawableTextUnderColor ( void )
@@ -1445,7 +1465,8 @@ Magick::PathArcAbs::PathArcAbs ( const PathArcArgsList &coordinates_ )
 {
 }
 Magick::PathArcAbs::PathArcAbs ( const Magick::PathArcAbs& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathArcAbs::~PathArcAbs ( void )
@@ -1475,7 +1496,8 @@ Magick::PathArcRel::PathArcRel ( const PathArcArgsList &coordinates_ )
 {
 }
 Magick::PathArcRel::PathArcRel ( const Magick::PathArcRel& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathArcRel::~PathArcRel ( void )
@@ -1590,8 +1612,9 @@ Magick::PathCurvetoAbs::PathCurvetoAbs ( const PathCurveToArgsList &args_ )
 {
 }
 Magick::PathCurvetoAbs::PathCurvetoAbs
-( const Magick::PathCurvetoAbs& original_ )
- : _args(original_._args)
+ ( const Magick::PathCurvetoAbs& original_ )
+   : VPathBase (original_),
+     _args(original_._args)
 {
 }
 Magick::PathCurvetoAbs::~PathCurvetoAbs ( void )
@@ -1621,7 +1644,8 @@ Magick::PathCurvetoRel::PathCurvetoRel ( const PathCurveToArgsList &args_ )
 }
 Magick::PathCurvetoRel::PathCurvetoRel
 ( const Magick::PathCurvetoRel& original_ )
-  : _args(original_._args)
+  : VPathBase (original_),
+    _args(original_._args)
 {
 }
 Magick::PathCurvetoRel::~PathCurvetoRel ( void )
@@ -1653,7 +1677,8 @@ Magick::PathSmoothCurvetoAbs::PathSmoothCurvetoAbs
 }
 Magick::PathSmoothCurvetoAbs::PathSmoothCurvetoAbs
 ( const Magick::PathSmoothCurvetoAbs& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathSmoothCurvetoAbs::~PathSmoothCurvetoAbs ( void )
@@ -1688,7 +1713,8 @@ Magick::PathSmoothCurvetoRel::PathSmoothCurvetoRel
 }
 Magick::PathSmoothCurvetoRel::PathSmoothCurvetoRel
 ( const Magick::PathSmoothCurvetoRel& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathSmoothCurvetoRel::~PathSmoothCurvetoRel ( void )
@@ -1795,7 +1821,8 @@ Magick::PathQuadraticCurvetoAbs::PathQuadraticCurvetoAbs
 }
 Magick::PathQuadraticCurvetoAbs::PathQuadraticCurvetoAbs
 ( const Magick::PathQuadraticCurvetoAbs& original_ )
-  : _args(original_._args)
+  : VPathBase (original_),
+    _args(original_._args)
 {
 }
 Magick::PathQuadraticCurvetoAbs::~PathQuadraticCurvetoAbs ( void )
@@ -1827,7 +1854,8 @@ Magick::PathQuadraticCurvetoRel::PathQuadraticCurvetoRel
 }
 Magick::PathQuadraticCurvetoRel::PathQuadraticCurvetoRel
 ( const Magick::PathQuadraticCurvetoRel& original_ )
-  : _args(original_._args)
+  : VPathBase (original_),
+    _args(original_._args)
 {
 }
 Magick::PathQuadraticCurvetoRel::~PathQuadraticCurvetoRel ( void )
@@ -1859,7 +1887,8 @@ Magick::PathSmoothQuadraticCurvetoAbs::PathSmoothQuadraticCurvetoAbs
 }
 Magick::PathSmoothQuadraticCurvetoAbs::PathSmoothQuadraticCurvetoAbs
 ( const Magick::PathSmoothQuadraticCurvetoAbs& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathSmoothQuadraticCurvetoAbs::~PathSmoothQuadraticCurvetoAbs ( void )
@@ -1890,7 +1919,8 @@ Magick::PathSmoothQuadraticCurvetoRel::PathSmoothQuadraticCurvetoRel
 }
 Magick::PathSmoothQuadraticCurvetoRel::PathSmoothQuadraticCurvetoRel
 ( const PathSmoothQuadraticCurvetoRel& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathSmoothQuadraticCurvetoRel::~PathSmoothQuadraticCurvetoRel ( void )
@@ -1922,7 +1952,8 @@ Magick::PathLinetoAbs::PathLinetoAbs ( const CoordinateList &coordinates_ )
 {
 }
 Magick::PathLinetoAbs::PathLinetoAbs ( const Magick::PathLinetoAbs& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathLinetoAbs::~PathLinetoAbs ( void )
@@ -1949,7 +1980,8 @@ Magick::PathLinetoRel::PathLinetoRel ( const CoordinateList &coordinates_ )
 {
 }
 Magick::PathLinetoRel::PathLinetoRel ( const Magick::PathLinetoRel& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathLinetoRel::~PathLinetoRel ( void )
@@ -2038,7 +2070,8 @@ Magick::PathMovetoAbs::PathMovetoAbs ( const CoordinateList &coordinates_ )
 {
 }
 Magick::PathMovetoAbs::PathMovetoAbs ( const Magick::PathMovetoAbs& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathMovetoAbs::~PathMovetoAbs ( void )
@@ -2065,7 +2098,8 @@ Magick::PathMovetoRel::PathMovetoRel ( const CoordinateList &coordinates_ )
 {
 }
 Magick::PathMovetoRel::PathMovetoRel ( const Magick::PathMovetoRel& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathMovetoRel::~PathMovetoRel ( void )
