@@ -1,6 +1,6 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
-// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002
+// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2008
 //
 // Inclusion of GraphicsMagick headers (with namespace magic)
 
@@ -18,20 +18,15 @@
 # undef class
 #endif
 
-// Needed for stdio FILE
 #include <stdio.h>
-
-// Needed for time_t
+#include <stdarg.h>
+#include <stdlib.h>
 #include <time.h>
 
 #if defined(HAVE_SYS_TYPES_H)
 # include <sys/types.h>
 #endif
 
-#if defined(macintosh)
-#  include <stat.mac.h>  /* Needed for off_t */
-#endif
-
 #if defined(__BORLANDC__)
 # include <vcl.h> /* Borland C++ Builder 4.0 requirement */
 #endif // defined(__BORLANDC__)
@@ -95,6 +90,16 @@ namespace MagickLib
 #  define MagickDLLDeclExtern
 #endif
 
+#if defined(WIN32) && defined(_VISUALC_)
+#  pragma warning(disable : 4996) /* function deprecation warnings */
+#endif
+
+#if defined(MAGICK_IMPLEMENTATION)
+namespace MagickLib
+{
+#  include "magick/enum_strings.h"
+}
+#endif
 
 //
 // Import GraphicsMagick symbols and types which are used as part of the
@@ -123,6 +128,8 @@ namespace Magick
   using MagickLib::OpacityChannel;
   using MagickLib::BlackChannel;
   using MagickLib::MatteChannel;
+  using MagickLib::AllChannels;
+  using MagickLib::GrayChannel;
   
   // Color-space types
   using MagickLib::ColorspaceType;
@@ -141,6 +148,11 @@ namespace Magick
   using MagickLib::sRGBColorspace;
   using MagickLib::HSLColorspace;
   using MagickLib::HWBColorspace;
+  using MagickLib::LABColorspace;
+  using MagickLib::CineonLogRGBColorspace;
+  using MagickLib::Rec601LumaColorspace;
+  using MagickLib::Rec709LumaColorspace;
+  using MagickLib::Rec709YCbCrColorspace;
   
   // Composition operations
   using MagickLib::AddCompositeOp;
@@ -180,6 +192,7 @@ namespace Magick
   using MagickLib::CopyMagentaCompositeOp;
   using MagickLib::CopyYellowCompositeOp;
   using MagickLib::CopyBlackCompositeOp;
+  using MagickLib::DivideCompositeOp;
   
   // Compression algorithms
   using MagickLib::CompressionType;
@@ -288,6 +301,18 @@ namespace Magick
   using MagickLib::ImpulseNoise;
   using MagickLib::LaplacianNoise;
   using MagickLib::PoissonNoise;
+
+  // Orientation types
+  using MagickLib::OrientationType;
+  using MagickLib::UndefinedOrientation;
+  using MagickLib::TopLeftOrientation;
+  using MagickLib::TopRightOrientation;
+  using MagickLib::BottomRightOrientation;
+  using MagickLib::BottomLeftOrientation;
+  using MagickLib::LeftTopOrientation;
+  using MagickLib::RightTopOrientation;
+  using MagickLib::RightBottomOrientation;
+  using MagickLib::LeftBottomOrientation;
   
   // Paint methods
   using MagickLib::PaintMethod;
@@ -300,13 +325,30 @@ namespace Magick
   // Arithmetic and bitwise operators
   using MagickLib::AddQuantumOp;
   using MagickLib::AndQuantumOp;
+  using MagickLib::AssignQuantumOp;
   using MagickLib::DivideQuantumOp;
   using MagickLib::LShiftQuantumOp;
   using MagickLib::MultiplyQuantumOp;
   using MagickLib::OrQuantumOp;
   using MagickLib::RShiftQuantumOp;
   using MagickLib::SubtractQuantumOp;
+  using MagickLib::ThresholdQuantumOp;
+  using MagickLib::ThresholdBlackQuantumOp;
+  using MagickLib::ThresholdWhiteQuantumOp;
   using MagickLib::XorQuantumOp;
+  using MagickLib::NoiseGaussianQuantumOp;
+  using MagickLib::NoiseImpulseQuantumOp;
+  using MagickLib::NoiseLaplacianQuantumOp;
+  using MagickLib::NoiseMultiplicativeQuantumOp;
+  using MagickLib::NoisePoissonQuantumOp;
+  using MagickLib::NoiseUniformQuantumOp;
+  using MagickLib::NegateQuantumOp;
+  using MagickLib::GammaQuantumOp;
+  using MagickLib::DepthQuantumOp;
+  using MagickLib::LogQuantumOp;
+  using MagickLib::MaxQuantumOp;
+  using MagickLib::MinQuantumOp;
+  using MagickLib::PowQuantumOp;
   using MagickLib::QuantumOperator;
 
   // Preview types.  Not currently used by Magick++
@@ -359,6 +401,14 @@ namespace Magick
   using MagickLib::RGBQuantum;
   using MagickLib::RGBAQuantum;
   using MagickLib::CMYKQuantum;
+  using MagickLib::CIEYQuantum;
+  using MagickLib::CIEXYZQuantum;
+
+  // Quantum sample types
+  using MagickLib::QuantumSampleType;
+  using MagickLib::UndefinedQuantumSampleType;
+  using MagickLib::UnsignedQuantumSampleType;
+  using MagickLib::FloatQuantumSampleType;
 
   // Rendering intents
   using MagickLib::RenderingIntent;
@@ -418,14 +468,22 @@ namespace Magick
   using MagickLib::OverlineDecoration;
   using MagickLib::LineThroughDecoration;
 
+  // Resource types
+  using MagickLib::ResourceType;
+  using MagickLib::FileResource;
+  using MagickLib::MemoryResource;
+  using MagickLib::MapResource;
+  using MagickLib::DiskResource;
+
 #if defined(MAGICK_IMPLEMENTATION)
   //
   // GraphicsMagick symbols used in implementation code
   //
   using MagickLib::AccessDefinition;
-  using MagickLib::AcquireCacheView;
+  using MagickLib::AccessImmutableIndexes;
+  using MagickLib::AccessMutableIndexes;
+  using MagickLib::AcquireCacheViewPixels;
   using MagickLib::AcquireImagePixels;
-  using MagickLib::AcquireMemory;
   using MagickLib::AdaptiveThresholdImage;
   using MagickLib::AddDefinitions;
   using MagickLib::AddNoiseImage;
@@ -578,10 +636,11 @@ namespace Magick
   using MagickLib::EdgeImage;
   using MagickLib::EmbossImage;
   using MagickLib::EnhanceImage;
-  using MagickLib::ExecuteModuleProcess;
   using MagickLib::EqualizeImage;
   using MagickLib::ExceptionInfo;
   using MagickLib::ExceptionType;
+  using MagickLib::ExecuteModuleProcess;
+  using MagickLib::ExportImagePixelArea;
   using MagickLib::FileOpenError;
   using MagickLib::FileOpenFatalError;
   using MagickLib::FileOpenWarning;
@@ -594,7 +653,8 @@ namespace Magick
   using MagickLib::GammaImage;
   using MagickLib::GammaImage;
   using MagickLib::GaussianBlurImage;
-  using MagickLib::GetCacheView;
+  using MagickLib::GetBlobSize;
+  using MagickLib::GetCacheViewPixels;
   using MagickLib::GetCacheViewIndexes;
   using MagickLib::GetColorTuple;
   using MagickLib::GetDrawInfo;
@@ -603,6 +663,7 @@ namespace Magick
   using MagickLib::GetImageAttribute;
   using MagickLib::GetImageBoundingBox;
   using MagickLib::GetImageChannelDepth;
+  using MagickLib::GetImageClipMask;
   using MagickLib::GetImageDepth;
   using MagickLib::GetImageInfo;
   using MagickLib::GetImagePixels;
@@ -610,14 +671,12 @@ namespace Magick
   using MagickLib::GetImageQuantizeError;
   using MagickLib::GetImageStatistics;
   using MagickLib::GetImageType;
-  using MagickLib::GetIndexes;
   using MagickLib::GetMagickGeometry;
   using MagickLib::GetMagickInfo;
   using MagickLib::GetMagickInfoArray;
   using MagickLib::GetMagickRegistry;
   using MagickLib::GetNumberColors;
   using MagickLib::GetPageGeometry;
-  using MagickLib::GetPixels;
   using MagickLib::GetQuantizeInfo;
   using MagickLib::GetTypeMetrics;
   using MagickLib::GlobExpression;
@@ -634,15 +693,20 @@ namespace Magick
   using MagickLib::ImageToBlob;
   using MagickLib::ImageWarning;
   using MagickLib::ImplodeImage;
+  using MagickLib::ImportImagePixelArea;
   using MagickLib::IsEventLogging;
   using MagickLib::IsGeometry;
   using MagickLib::IsImagesEqual;
   using MagickLib::IsSubimage;
   using MagickLib::LessValue;
-  using MagickLib::LiberateMemory;
+  using MagickLib::LevelImage;
+  using MagickLib::LevelImageChannel;
   using MagickLib::LocaleCompare;
   using MagickLib::LogMagickEvent;
+  using MagickLib::MagickFree;
   using MagickLib::MagickInfo;
+  using MagickLib::MagickMalloc;
+  using MagickLib::MagickRealloc;
   using MagickLib::MagickToMime;
   using MagickLib::MagnifyImage;
   using MagickLib::MapImage;
@@ -685,7 +749,6 @@ namespace Magick
   using MagickLib::QueryColorDatabase;
   using MagickLib::RGBTransformImage;
   using MagickLib::RaiseImage;
-  using MagickLib::ReacquireMemory;
   using MagickLib::ReadImage;
   using MagickLib::RectangleInfo;
   using MagickLib::RectangleInfo;
@@ -704,8 +767,7 @@ namespace Magick
   using MagickLib::SampleImage;
   using MagickLib::ScaleImage;
   using MagickLib::SegmentImage;
-  using MagickLib::SetCacheThreshold;
-  using MagickLib::SetCacheView;
+  using MagickLib::SetCacheViewPixels;
   using MagickLib::SetClientName;
   using MagickLib::SetImage;
   using MagickLib::SetImageAttribute;
@@ -720,12 +782,13 @@ namespace Magick
   using MagickLib::SetLogEventMask;
   using MagickLib::SetMagickInfo;
   using MagickLib::SetMagickRegistry;
+  using MagickLib::SetMagickResourceLimit;
+  using MagickLib::SetMagickResourceLimit;
   using MagickLib::ShadeImage;
   using MagickLib::SharpenImage;
   using MagickLib::ShaveImage;
   using MagickLib::ShearImage;
   using MagickLib::SignatureImage;
-  using MagickLib::SizeBlob;
   using MagickLib::SolarizeImage;
   using MagickLib::SpreadImage;
   using MagickLib::SteganoImage;
@@ -734,7 +797,7 @@ namespace Magick
   using MagickLib::StreamFatalError;
   using MagickLib::StreamWarning;
   using MagickLib::SwirlImage;
-  using MagickLib::SyncCacheView;
+  using MagickLib::SyncCacheViewPixels;
   using MagickLib::SyncImage;
   using MagickLib::SyncImagePixels;
   using MagickLib::TextureImage;
@@ -765,6 +828,15 @@ namespace Magick
   using MagickLib::YValue;
   using MagickLib::ZoomImage;
 
+  using MagickLib::AddNoiseImageChannel;
+  using MagickLib::BlurImageChannel;
+  using MagickLib::GaussianBlurImageChannel;
+  using MagickLib::MotionBlurImage;
+  using MagickLib::RandomChannelThresholdImage;
+  using MagickLib::SharpenImageChannel;
+  using MagickLib::UnsharpMaskImageChannel;
+
+
 #endif // MAGICK_IMPLEMENTATION
 
 }
