@@ -37,11 +37,12 @@
 */
 #include "magick/studio.h"
 #include "magick/attribute.h"
-#include "magick/cache.h"
+#include "magick/pixel_cache.h"
 #include "magick/blob.h"
 #include "magick/color.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
+#include "magick/profile.h"
 #include "magick/utility.h"
 
 /*
@@ -153,6 +154,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned long
     quantum_depth;
 
+  ProfileInfo
+    *profiles=0;
+
+  unsigned int
+    number_of_profiles=0;
+
   /*
     Open image file.
   */
@@ -164,7 +171,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
     ThrowReaderException(FileOpenError,UnableToOpenFile,image);
-  (void) strncpy(cache_filename,image->filename,MaxTextExtent-1);
+  (void) strlcpy(cache_filename,image->filename,MaxTextExtent);
   AppendImageFormat("cache",cache_filename);
   c=ReadBlobByte(image);
   if (c == EOF)
@@ -205,11 +212,11 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             c=ReadBlobByte(image);
             if ((c == EOF) || (c == '}'))
               break;
-            if ((p-comment+1) >= length)
+            if ((unsigned long) (p-comment+1) >= length)
               {
                 *p='\0';
                 length<<=1;
-                MagickReallocMemory(comment,length);
+                MagickReallocMemory(char *,comment,length);
                 if (comment == (char *) NULL)
                   break;
                 p=comment+strlen(comment);
@@ -247,7 +254,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 {
                   *p='\0';
                   length<<=1;
-                  MagickReallocMemory(values,length);
+                  MagickReallocMemory(char *,values,length);
                   if (values == (char *) NULL)
                     break;
                   p=values+strlen(values);
@@ -428,7 +435,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"id") == 0)
                   {
-                    (void) strncpy(id,values,MaxTextExtent-1);
+                    (void) strlcpy(id,values,MaxTextExtent);
                     break;
                   }
                 if (LocaleCompare(keyword,"iterations") == 0)
@@ -496,29 +503,14 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleNCompare(keyword,"profile-",8) == 0)
                   {
-                    if (LocaleCompare(keyword,"profile-icc") == 0)
-                      {
-                        image->color_profile.length=atol(values);
-                        break;
-                      }
-                    if (LocaleCompare(keyword,"profile-iptc") == 0)
-                      {
-                        image->iptc_profile.length=atol(values);
-                        break;
-                      }
-                    i=(long) image->generic_profiles;
-                    if (image->generic_profile == (ProfileInfo *) NULL)
-                      image->generic_profile=MagickAllocateMemory(ProfileInfo *,
-                        sizeof(ProfileInfo));
-                    else
-                      MagickReallocMemory(image->generic_profile,
-                        (i+1)*sizeof(ProfileInfo));
-                    if (image->generic_profile == (ProfileInfo *) NULL)
+                    i=(long) number_of_profiles;
+                    MagickReallocMemory(ProfileInfo *,profiles,(i+1)*sizeof(ProfileInfo));
+                    if (profiles == (ProfileInfo *) NULL)
                       ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
-                    image->generic_profile[i].name=AllocateString(keyword+8);
-                    image->generic_profile[i].length=atol(values);
-                    image->generic_profile[i].info=(unsigned char *) NULL;
-                    image->generic_profiles++;
+                    profiles[i].name=AllocateString(keyword+8);
+                    profiles[i].length=atol(values);
+                    profiles[i].info=(unsigned char *) NULL;
+                    number_of_profiles++;
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -579,7 +571,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if (LocaleCompare(keyword,"resolution") == 0)
                   {
                     (void) GetMagickDimension(values,&image->x_resolution,
-                      &image->y_resolution);
+                                              &image->y_resolution,NULL,NULL);
                     break;
                   }
                 if (LocaleCompare(keyword,"rows") == 0)
@@ -679,7 +671,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 Allocate more memory for the image directory.
               */
-              MagickReallocMemory(image->directory,
+              MagickReallocMemory(char *,image->directory,
                 (strlen(image->directory)+MaxTextExtent+1));
               if (image->directory == (char *) NULL)
                 ThrowReaderException(CorruptImageError,UnableToReadImageData,
@@ -690,50 +682,30 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           *p++=c;
         } while (c != '\0');
       }
-    if (image->color_profile.length != 0)
+
+    /*
+      Attached profiles.
+    */
+    if (number_of_profiles > 0)
       {
-        /*
-          Color profile.
-        */
-        image->color_profile.info=MagickAllocateMemory(unsigned char *,
-          image->color_profile.length);
-        if (image->color_profile.info == (unsigned char *) NULL)
-          ThrowReaderException(CorruptImageError,UnableToReadColorProfile,
-            image);
-        (void) ReadBlob(image,image->color_profile.length,
-          image->color_profile.info);
-      }
-    if (image->iptc_profile.length != 0)
-      {
-        /*
-          IPTC profile.
-        */
-        image->iptc_profile.info=MagickAllocateMemory(unsigned char *,
-          image->iptc_profile.length);
-        if (image->iptc_profile.info == (unsigned char *) NULL)
-          ThrowReaderException(CorruptImageError,UnableToReadIPTCProfile,
-            image);
-        (void) ReadBlob(image,image->iptc_profile.length,
-          image->iptc_profile.info);
-      }
-    if (image->generic_profiles != 0)
-      {
-        /*
-          Generic profile.
-        */
-        for (i=0; i < (long) image->generic_profiles; i++)
+        for (i=0; i < (long) number_of_profiles; i++)
         {
-          if (image->generic_profile[i].length == 0)
+          if (profiles[i].length == 0)
             continue;
-          image->generic_profile[i].info=MagickAllocateMemory(unsigned char *,
-            image->generic_profile[i].length);
-          if (image->generic_profile[i].info == (unsigned char *) NULL)
+          profiles[i].info=MagickAllocateMemory(unsigned char *,profiles[i].length);
+          if (profiles[i].info == (unsigned char *) NULL)
             ThrowReaderException(CorruptImageError,UnableToReadGenericProfile,
               image);
-          (void) ReadBlob(image,image->generic_profile[i].length,
-            image->generic_profile[i].info);
+          (void) ReadBlob(image,profiles[i].length,profiles[i].info);
+          (void) SetImageProfile(image,profiles[i].name,profiles[i].info,
+                                 profiles[i].length);
+          MagickFreeMemory(profiles[i].name);
+          MagickFreeMemory(profiles[i].info);
         }
+        MagickFreeMemory(profiles);
+        number_of_profiles=0;
       }
+
     if (image->storage_class == PseudoClass)
       {
         /*
@@ -822,8 +794,9 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image),
-          exception);
+        status=MagickMonitorFormatted(TellBlob(image),GetBlobSize(image),
+                                      exception,LoadImagesText,
+                                      image->filename);
         if (status == False)
           break;
       }
@@ -863,15 +836,18 @@ ModuleExport void RegisterMPCImage(void)
     *entry;
 
   entry=SetMagickInfo("CACHE");
-  entry->description=AcquireString("Magick Persistent Cache image format");
-  entry->module=AcquireString("CACHE");
+  entry->description="Magick Persistent Cache image format";
+  entry->module="CACHE";
+  entry->coder_class=UnstableCoderClass;
   (void) RegisterMagickInfo(entry);
+
   entry=SetMagickInfo("MPC");
   entry->decoder=(DecoderHandler) ReadMPCImage;
   entry->encoder=(EncoderHandler) WriteMPCImage;
   entry->magick=(MagickHandler) IsMPC;
-  entry->description=AcquireString("Magick Persistent Cache image format");
-  entry->module=AcquireString("MPC");
+  entry->description="Magick Persistent Cache image format";
+  entry->module="MPC";
+  entry->coder_class=UnstableCoderClass;
   (void) RegisterMagickInfo(entry);
 }
 
@@ -950,6 +926,18 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
   unsigned long
     scene;
 
+  ImageProfileIterator
+    profile_iterator;
+  
+  const char
+    *profile_name;
+  
+  const unsigned char
+    *profile_info;
+  
+  size_t
+    profile_length;
+
   /*
     Open persistent cache.
   */
@@ -960,7 +948,7 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
     ThrowWriterException(FileOpenError,UnableToOpenFile,image);
-  (void) strncpy(cache_filename,image->filename,MaxTextExtent-1);
+  (void) strlcpy(cache_filename,image->filename,MaxTextExtent);
   AppendImageFormat("cache",cache_filename);
   scene=0;
   offset=0;
@@ -973,15 +961,15 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
     FormatString(buffer,"quantum-depth=%d\n",QuantumDepth);
     (void) WriteBlobString(image,buffer);
     if (image->storage_class == PseudoClass)
-      FormatString(buffer,"class=PseudoClass  colors=%lu  matte=%s\n",
-        image->colors,image->matte ? "True" : "False");
+      FormatString(buffer,"class=PseudoClass  colors=%u  matte=%s\n",
+                   image->colors,MagickBoolToString(image->matte));
     else
       if (image->colorspace == CMYKColorspace)
         FormatString(buffer,"class=DirectClass  colorspace=CMYK  matte=%s\n",
-          image->matte ? "True" : "False");
+                     MagickBoolToString(image->matte));
       else
         FormatString(buffer,"class=DirectClass  matte=%s\n",
-          image->matte ? "True" : "False");
+                     MagickBoolToString(image->matte));
     (void) WriteBlobString(image,buffer);
     switch (image->compression)
     {
@@ -1032,7 +1020,7 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
         break;
       }
     }
-    FormatString(buffer,"columns=%lu  rows=%lu  depth=%lu\n",image->columns,
+    FormatString(buffer,"columns=%lu  rows=%lu  depth=%u\n",image->columns,
       image->rows,image->depth);
     (void) WriteBlobString(image,buffer);
     if (image->is_monochrome != MagickFalse)
@@ -1149,32 +1137,25 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
           image->chromaticity.white_point.x,image->chromaticity.white_point.y);
         (void) WriteBlobString(image,buffer);
       }
-    if (image->color_profile.length != 0)
+    /*
+      Attached profiles.
+    */    
+    profile_iterator=AllocateImageProfileIterator(image);
+    if (profile_iterator)
       {
-        FormatString(buffer,"profile-icc=%lu\n",(unsigned long)
-          image->color_profile.length);
-        (void) WriteBlobString(image,buffer);
-      }
-    if (image->iptc_profile.length != 0)
-      {
-        FormatString(buffer,"profile-iptc=%lu\n",(unsigned long)
-          image->iptc_profile.length);
-        (void) WriteBlobString(image,buffer);
-      }
-    if (image->generic_profiles != 0)
-      {
-        /*
-          Generic profile.
-        */
-        for (i=0; i < (long) image->generic_profiles; i++)
-        {
-          FormatString(buffer,"profile-%s=%lu\n",
-            image->generic_profile[i].name == (char *) NULL ? "generic" :
-            image->generic_profile[i].name,(unsigned long)
-            image->generic_profile[i].length);
-          (void) WriteBlobString(image,buffer);
-        }
+        while(NextImageProfile(profile_iterator,&profile_name,&profile_info,
+                               &profile_length)
+              != MagickFail)
+          {
+            FormatString(buffer,"profile-%.1024s=%lu\n",
+                         profile_name == (char *) NULL ? "generic" :
+                         profile_name,(unsigned long)
+                         profile_length);
+            (void) WriteBlobString(image,buffer);
+          }
+        DeallocateImageProfileIterator(profile_iterator);
       }
+
     if (image->montage != (char *) NULL)
       {
         FormatString(buffer,"montage=%.1024s\n",image->montage);
@@ -1211,25 +1192,22 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlobString(image,image->directory);
         (void) WriteBlobByte(image,'\0');
       }
-    if (image->color_profile.length != 0)
-      (void) WriteBlob(image,image->color_profile.length,
-        image->color_profile.info);
-    if (image->iptc_profile.length != 0)
-      (void) WriteBlob(image,image->iptc_profile.length,
-        image->iptc_profile.info);
-    if (image->generic_profiles != 0)
+
+    /*
+      Attached profiles.
+    */
+    profile_iterator=AllocateImageProfileIterator(image);
+    if (profile_iterator)
       {
-        /*
-          Generic profile.
-        */
-        for (i=0; i < (long) image->generic_profiles; i++)
-        {
-          if (image->generic_profile[i].length == 0)
-            continue;
-          (void) WriteBlob(image,image->generic_profile[i].length,
-            image->generic_profile[i].info);
-        }
+        while(NextImageProfile(profile_iterator,&profile_name,&profile_info,
+                               &profile_length) != MagickFail)
+          {
+            if (profile_length != 0)
+              (void) WriteBlob(image,profile_length,profile_info);
+          }
+        DeallocateImageProfileIterator(profile_iterator);
       }
+
     if (image->storage_class == PseudoClass)
       {
         register unsigned char
@@ -1259,6 +1237,7 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
             *q++=image->colormap[i].green;
             *q++=image->colormap[i].blue;
           }
+#if QuantumDepth > 8
         else
           for (i=0; i < (long) image->colors; i++)
           {
@@ -1269,6 +1248,8 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
             *q++=image->colormap[i].blue >> 8;
             *q++=image->colormap[i].blue & 0xff;
           }
+#endif /* QuantumDepth > 8 */
+
         (void) WriteBlob(image,packet_size*image->colors,colormap);
         MagickFreeMemory(colormap);
       }
@@ -1281,8 +1262,9 @@ static unsigned int WriteMPCImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesText,scene++,GetImageListLength(image),
-      &image->exception);
+    status=MagickMonitorFormatted(scene++,GetImageListLength(image),
+                                  &image->exception,SaveImagesText,
+                                  image->filename);
     if (status == False)
       break;
   } while (image_info->adjoin);
