@@ -37,7 +37,7 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
+#include "magick/pixel_cache.h"
 #include "magick/constitute.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
@@ -104,8 +104,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned int
     status;
 
-  unsigned long
-    packet_size;
+  unsigned int
+    packet_size,
+    quantum_size;
 
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
@@ -123,16 +124,27 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (status == False)
         ThrowReaderException(FileOpenError,UnableToOpenFile,image);
       for (i=0; i < image->offset; i++)
-        (void) ReadBlobByte(image);
+        {
+          if (EOF == ReadBlobByte(image))
+            ThrowException(exception,CorruptImageError,UnexpectedEndOfFile,
+                           image->filename);
+        }
     }
   /*
     Allocate memory for a scanline.
   */
-  packet_size=image->depth > 8 ? 6 : 3;
+  if (image->depth <= 8)
+    quantum_size=8;
+  else if (image->depth <= 16)
+    quantum_size=16;
+  else
+    quantum_size=32;
+
+  packet_size=(quantum_size*3)/8;
   if (LocaleCompare(image_info->magick,"RGBA") == 0)
     {
       image->matte=True;
-      packet_size=image->depth > 8 ? 8 : 4;
+      packet_size=(quantum_size*4)/8;
     }
   scanline=MagickAllocateMemory(unsigned char *,
     packet_size*image->tile_info.width);
@@ -175,14 +187,15 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (q == (PixelPacket *) NULL)
             break;
           if (!image->matte)
-            (void) PushImagePixels(image,RGBQuantum,scanline+x);
+            (void) ImportImagePixelArea(image,RGBQuantum,quantum_size,scanline+x,0,0);
           else
-            (void) PushImagePixels(image,RGBAQuantum,scanline+x);
+            (void) ImportImagePixelArea(image,RGBAQuantum,quantum_size,scanline+x,0,0);
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+              if (!MagickMonitorFormatted(y,image->rows,exception,
+                                          LoadImageText,image->filename))
                 break;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
@@ -195,7 +208,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Line interlacing:  RRR...GGG...BBB...RRR...GGG...BBB...
         */
-        packet_size=image->depth > 8 ? 2 : 1;
+        packet_size=(quantum_size)/8;
         for (y=0; y < image->tile_info.y; y++)
           (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
         for (y=0; y < (long) image->rows; y++)
@@ -205,22 +218,23 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,RedQuantum,scanline+x);
+          (void) ImportImagePixelArea(image,RedQuantum,quantum_size,scanline+x,0,0);
           (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
-          (void) PushImagePixels(image,GreenQuantum,scanline+x);
+          (void) ImportImagePixelArea(image,GreenQuantum,quantum_size,scanline+x,0,0);
           (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
-          (void) PushImagePixels(image,BlueQuantum,scanline+x);
+          (void) ImportImagePixelArea(image,BlueQuantum,quantum_size,scanline+x,0,0);
           if (image->matte)
             {
               (void) ReadBlob(image,packet_size*image->tile_info.width,
                 scanline);
-              (void) PushImagePixels(image,AlphaQuantum,scanline+x);
+              (void) ImportImagePixelArea(image,AlphaQuantum,quantum_size,scanline+x,0,0);
             }
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+              if (!MagickMonitorFormatted(y,image->rows,exception,
+                                          LoadImageText,image->filename))
                 break;
         }
         count=image->tile_info.height-image->rows-image->tile_info.y;
@@ -244,7 +258,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (status == False)
               ThrowReaderException(FileOpenError,UnableToOpenFile,image);
           }
-        packet_size=image->depth > 8 ? 2 : 1;
+        packet_size=(quantum_size)/8;
         for (y=0; y < image->tile_info.y; y++)
           (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
         i=0;
@@ -256,12 +270,13 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,RedQuantum,scanline+x);
+          (void) ImportImagePixelArea(image,RedQuantum,quantum_size,scanline+x,0,0);
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              if (!MagickMonitor(LoadImageText,i,span,&image->exception))
+              if (!MagickMonitorFormatted(i,span,&image->exception,
+                                          LoadImageText,image->filename))
                 break;
           i++;
         }
@@ -284,12 +299,13 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,GreenQuantum,scanline+x);
+          (void) ImportImagePixelArea(image,GreenQuantum,quantum_size,scanline+x,0,0);
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              if (!MagickMonitor(LoadImageText,i,span,&image->exception))
+              if (!MagickMonitorFormatted(i,span,&image->exception,
+                                          LoadImageText,image->filename))
                 break;
           i++;
         }
@@ -312,12 +328,13 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) PushImagePixels(image,BlueQuantum,scanline+x);
+          (void) ImportImagePixelArea(image,BlueQuantum,quantum_size,scanline+x,0,0);
           if (!SyncImagePixels(image))
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              if (!MagickMonitor(LoadImageText,i,span,&image->exception))
+              if (!MagickMonitorFormatted(i,span,&image->exception,
+                                          LoadImageText,image->filename))
                 break;
           i++;
         }
@@ -347,12 +364,13 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               q=GetImagePixels(image,0,y,image->columns,1);
               if (q == (PixelPacket *) NULL)
                 break;
-              (void) PushImagePixels(image,AlphaQuantum,scanline+x);
+              (void) ImportImagePixelArea(image,AlphaQuantum,quantum_size,scanline+x,0,0);
               if (!SyncImagePixels(image))
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(i,span))
-                  if (!MagickMonitor(LoadImageText,i,span,&image->exception))
+                  if (!MagickMonitorFormatted(i,span,&image->exception,
+                                              LoadImageText,image->filename))
                     break;
               i++;
             }
@@ -362,7 +380,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 scanline);
           }
         if (image_info->interlace == PartitionInterlace)
-          (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
+          (void) strlcpy(image->filename,image_info->filename,MaxTextExtent);
         break;
       }
     }
@@ -393,7 +411,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (!MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image),exception))
+        if (!MagickMonitorFormatted(TellBlob(image),GetBlobSize(image),
+                                    exception,LoadImagesText,
+                                    image->filename))
           break;
       }
   } while (count != 0);
@@ -436,15 +456,16 @@ ModuleExport void RegisterRGBImage(void)
   entry->decoder=(DecoderHandler) ReadRGBImage;
   entry->encoder=(EncoderHandler) WriteRGBImage;
   entry->raw=True;
-  entry->description=AcquireString("Raw red, green, and blue samples");
-  entry->module=AcquireString("RGB");
+  entry->description="Raw red, green, and blue samples";
+  entry->module="RGB";
   (void) RegisterMagickInfo(entry);
+
   entry=SetMagickInfo("RGBA");
   entry->decoder=(DecoderHandler) ReadRGBImage;
   entry->encoder=(EncoderHandler) WriteRGBImage;
   entry->raw=True;
-  entry->description=AcquireString("Raw red, green, blue, and matte samples");
-  entry->module=AcquireString("RGB");
+  entry->description="Raw red, green, blue, and matte samples";
+  entry->module="RGB";
   (void) RegisterMagickInfo(entry);
 }
 
@@ -517,8 +538,9 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
   unsigned int
     status;
 
-  unsigned long
+  unsigned int
     packet_size,
+    quantum_size,
     scene;
 
   /*
@@ -528,9 +550,17 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  packet_size=image->depth > 8 ? 6 : 3;
+
+  if (image->depth <= 8)
+    quantum_size=8;
+  else if (image->depth <= 16)
+    quantum_size=16;
+  else
+    quantum_size=32;
+
+  packet_size=(quantum_size*3)/8;
   if (LocaleCompare(image_info->magick,"RGBA") == 0)
-    packet_size=image->depth > 8 ? 8 : 4;
+    packet_size=(quantum_size*4)/8;
   pixels=MagickAllocateMemory(unsigned char *,packet_size*image->columns);
   if (pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
@@ -549,7 +579,7 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
     /*
       Convert MIFF to RGB raster pixels.
     */
-    TransformColorspace(image,RGBColorspace);
+    (void) TransformColorspace(image,RGBColorspace);
     if (LocaleCompare(image_info->magick,"RGBA") == 0)
       if (!image->matte)
         SetImageOpacity(image,OpaqueOpacity);
@@ -568,17 +598,18 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
             break;
           if (LocaleCompare(image_info->magick,"RGBA") != 0)
             {
-              (void) PopImagePixels(image,RGBQuantum,pixels);
+              (void) ExportImagePixelArea(image,RGBQuantum,quantum_size,pixels,0,0);
               (void) WriteBlob(image,packet_size*image->columns,pixels);
             }
           else
             {
-              (void) PopImagePixels(image,RGBAQuantum,pixels);
+              (void) ExportImagePixelArea(image,RGBAQuantum,quantum_size,pixels,0,0);
               (void) WriteBlob(image,packet_size*image->columns,pixels);
             }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+              if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                          SaveImageText,image->filename))
                 break;
         }
         break;
@@ -593,19 +624,20 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,RedQuantum,pixels);
+          (void) ExportImagePixelArea(image,RedQuantum,quantum_size,pixels,0,0);
           (void) WriteBlob(image,image->columns,pixels);
-          (void) PopImagePixels(image,GreenQuantum,pixels);
+          (void) ExportImagePixelArea(image,GreenQuantum,quantum_size,pixels,0,0);
           (void) WriteBlob(image,image->columns,pixels);
-          (void) PopImagePixels(image,BlueQuantum,pixels);
+          (void) ExportImagePixelArea(image,BlueQuantum,quantum_size,pixels,0,0);
           (void) WriteBlob(image,image->columns,pixels);
           if (LocaleCompare(image_info->magick,"RGBA") == 0)
             {
-              (void) PopImagePixels(image,AlphaQuantum,pixels);
+              (void) ExportImagePixelArea(image,AlphaQuantum,quantum_size,pixels,0,0);
               (void) WriteBlob(image,image->columns,pixels);
             }
           if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+            if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                        SaveImageText,image->filename))
               break;
         }
         break;
@@ -628,7 +660,7 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,RedQuantum,pixels);
+          (void) ExportImagePixelArea(image,RedQuantum,quantum_size,pixels,0,0);
           (void) WriteBlob(image,image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
@@ -639,14 +671,15 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
             if (status == False)
               ThrowWriterException(FileOpenError,UnableToOpenFile,image);
           }
-        if (!MagickMonitor(SaveImageText,100,400,&image->exception))
+        if (!MagickMonitorFormatted(100,400,&image->exception,SaveImageText,
+                                    image->filename))
           break;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,GreenQuantum,pixels);
+          (void) ExportImagePixelArea(image,GreenQuantum,quantum_size,pixels,0,0);
           (void) WriteBlob(image,image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
@@ -657,19 +690,21 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
             if (status == False)
               ThrowWriterException(FileOpenError,UnableToOpenFile,image);
           }
-        if (!MagickMonitor(SaveImageText,200,400,&image->exception))
+        if (!MagickMonitorFormatted(200,400,&image->exception,SaveImageText,
+                                    image->filename))
           break;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          (void) PopImagePixels(image,BlueQuantum,pixels);
+          (void) ExportImagePixelArea(image,BlueQuantum,quantum_size,pixels,0,0);
           (void) WriteBlob(image,image->columns,pixels);
         }
         if (LocaleCompare(image_info->magick,"RGBA") == 0)
           {
-            if (!MagickMonitor(SaveImageText,300,400,&image->exception))
+            if (!MagickMonitorFormatted(300,400,&image->exception,
+                                        SaveImageText,image->filename))
               break;
             if (image_info->interlace == PartitionInterlace)
               {
@@ -686,13 +721,14 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              (void) PopImagePixels(image,AlphaQuantum,pixels);
+              (void) ExportImagePixelArea(image,AlphaQuantum,quantum_size,pixels,0,0);
               (void) WriteBlob(image,image->columns,pixels);
             }
           }
         if (image_info->interlace == PartitionInterlace)
-          (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
-        if (!MagickMonitor(SaveImageText,400,400,&image->exception))
+          (void) strlcpy(image->filename,image_info->filename,MaxTextExtent);
+        if (!MagickMonitorFormatted(400,400,&image->exception,SaveImageText,
+                                    image->filename))
           break;
         break;
       }
@@ -700,7 +736,9 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (!MagickMonitor(SaveImagesText,scene++,GetImageListLength(image),&image->exception))
+    if (!MagickMonitorFormatted(scene++,GetImageListLength(image),
+                                &image->exception,SaveImagesText,
+                                image->filename))
       break;
   } while (image_info->adjoin);
   MagickFreeMemory(pixels);
