@@ -37,7 +37,7 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
+#include "magick/pixel_cache.h"
 #include "magick/color.h"
 #include "magick/constitute.h"
 #include "magick/magick.h"
@@ -105,8 +105,8 @@ static unsigned int IsTXT(const unsigned char *magick,const size_t length)
     char
       buffer[MaxTextExtent];
 
-    memset((void *)buffer,0,MaxTextExtent);
-    memcpy((void *)buffer,(const void *)magick,Min(MaxTextExtent,length));
+    (void) memset((void *)buffer,0,MaxTextExtent);
+    (void) memcpy((void *)buffer,(const void *)magick,Min(MaxTextExtent,length));
 
     count=sscanf(buffer,"%lu,%lu: (%u, %u, %u) #%02X%02X%02X",
                  &column, &row, &red, &green, &blue, &hex_red, &hex_green,
@@ -239,7 +239,7 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
       (void) strcpy(density,PSDensityGeometry);
       count=GetMagickDimension(density,&image->x_resolution,
-        &image->y_resolution);
+        &image->y_resolution,NULL,NULL);
       if (count != 2)
         image->y_resolution=image->x_resolution;
     }
@@ -266,14 +266,14 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       clone_info=CloneImageInfo(image_info);
       clone_info->blob=(void *) NULL;
       clone_info->length=0;
-      (void) strncpy(clone_info->filename,image_info->texture,MaxTextExtent-1);
+      (void) strlcpy(clone_info->filename,image_info->texture,MaxTextExtent);
       texture=ReadImage(clone_info,exception);
       DestroyImageInfo(clone_info);
     }
   /*
     Annotate the text image.
   */
-  SetImage(image,OpaqueOpacity);
+  (void) SetImage(image,OpaqueOpacity);
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
   draw_info->fill=image_info->pen;
   (void) CloneString(&draw_info->text,image_info->filename);
@@ -282,7 +282,7 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=GetTypeMetrics(image,draw_info,&metrics);
   if (status == False)
     ThrowReaderException(TypeError,UnableToGetTypeMetrics,image);
-  (void) strncpy(filename,image_info->filename,MaxTextExtent-1);
+  (void) strlcpy(filename,image_info->filename,MaxTextExtent);
   if (draw_info->text != '\0')
     *draw_info->text='\0';
   p=ReadBlobString(image,text);
@@ -302,7 +302,8 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     offset+=(long) (metrics.ascent-metrics.descent);
     if (image->previous == (Image *) NULL)
       if (QuantumTick(offset,image->rows))
-        if (!MagickMonitor(LoadImageText,offset,image->rows,&image->exception))
+        if (!MagickMonitorFormatted(offset,image->rows,&image->exception,
+                                    LoadImageText,image->filename))
           break;
     p=ReadBlobString(image,text);
     if ((offset < (long) image->rows) && (p != (char *) NULL))
@@ -313,7 +314,7 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
           handler;
 
         handler=SetMonitorHandler((MonitorHandler) NULL);
-        TextureImage(image,texture);
+        (void) TextureImage(image,texture);
         (void) SetMonitorHandler(handler);
       }
     (void) AnnotateImage(image,draw_info);
@@ -333,9 +334,10 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->next->columns=image->columns;
     image->next->rows=image->rows;
     image=SyncNextImageInList(image);
-    (void) strncpy(image->filename,filename,MaxTextExtent-1);
-    SetImage(image,OpaqueOpacity);
-    if (!MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image),exception))
+    (void) strlcpy(image->filename,filename,MaxTextExtent);
+    (void) SetImage(image,OpaqueOpacity);
+    if (!MagickMonitorFormatted(TellBlob(image),GetBlobSize(image),exception,
+                                LoadImagesText,image->filename))
       break;
   }
   if (texture != (Image *) NULL)
@@ -344,7 +346,7 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
         handler;
 
       handler=SetMonitorHandler((MonitorHandler) NULL);
-      TextureImage(image,texture);
+      (void) TextureImage(image,texture);
       (void) SetMonitorHandler(handler);
     }
   (void) AnnotateImage(image,draw_info);
@@ -389,15 +391,15 @@ ModuleExport void RegisterTXTImage(void)
   entry->decoder=(DecoderHandler) ReadTXTImage;
   entry->encoder=(EncoderHandler) WriteTXTImage;
   entry->raw=True;
-  entry->description=AcquireString("Text");
-  entry->module=AcquireString("TXT");
+  entry->description="Text";
+  entry->module="TXT";
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("TXT");
   entry->decoder=(DecoderHandler) ReadTXTImage;
   entry->encoder=(EncoderHandler) WriteTXTImage;
-  entry->description=AcquireString("Text");
-  entry->module=AcquireString("TXT");
+  entry->description="Text";
+  entry->module="TXT";
   (void) RegisterMagickInfo(entry);
 }
 
@@ -492,7 +494,7 @@ static unsigned int WriteTXTImage(const ImageInfo *image_info,Image *image)
     /*
       Convert MIFF to TXT raster pixels.
     */
-    TransformColorspace(image,RGBColorspace);
+    (void) TransformColorspace(image,RGBColorspace);
     for (y=0; y < (long) image->rows; y++)
     {
       p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -514,8 +516,9 @@ static unsigned int WriteTXTImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesText,scene++,GetImageListLength(image),
-      &image->exception);
+    status=MagickMonitorFormatted(scene++,GetImageListLength(image),
+                                  &image->exception,SaveImagesText,
+                                  image->filename);
     if (status == False)
       break;
   } while (image_info->adjoin);
