@@ -44,7 +44,8 @@
 #include "magick/blob.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
-#if defined(WIN32) || defined(__CYGWIN__)
+#include "magick/log.h"
+#if defined(MSWINDOWS) || defined(__CYGWIN__)
 # include "magick/nt_feature.h"
 #endif
 #if defined(POSIX)
@@ -314,6 +315,64 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t P o s t s c r i p t D e l e g a t e I n f o                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method GetPostscriptDelegateInfo returns the Postscript delegate which
+%  best supports the image type requested via ImageInfo
+%
+%  The format of the GetPostscriptDelegateInfo method is:
+%
+%      const DelegateInfo *(const ImageInfo *image_info,
+%                           ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image_info: The 'monochrome' and 'type' fields of image_info are used
+%                  to select the best postscript delegate type.
+%
+%    o antialias: Set to best antialias setting for this delegate based on
+%                 user requested antialias setting, and rendering depth.
+%
+%    o exception: Return any errors or warnings in this structure.
+%
+%
+*/
+MagickExport const DelegateInfo *GetPostscriptDelegateInfo(const ImageInfo *image_info,
+                                                           unsigned int *antialias,
+                                                           ExceptionInfo *exception)
+{
+  char
+    delegate[MaxTextExtent];
+
+  (void) strlcpy(delegate,"gs-color",sizeof(delegate));
+  *antialias=(image_info->antialias ? 4 : 1);
+  if ((image_info->monochrome) || (BilevelType == image_info->type))
+    {
+      (void) strlcpy(delegate,"gs-mono",sizeof(delegate));
+      *antialias=1;
+    }
+  else if (GrayscaleType == image_info->type)
+    {
+      (void) strlcpy(delegate,"gs-gray",sizeof(delegate));
+    }
+  else if ((GrayscaleMatteType == image_info->type) ||
+           (PaletteMatteType == image_info->type) ||
+           (TrueColorMatteType == image_info->type))
+    {
+      (void) strlcpy(delegate,"gs-color+alpha",sizeof(delegate));
+    }
+  return GetDelegateInfo(delegate,(char *) NULL,exception);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   I n v o k e D e l e g a t e                                               %
 %                                                                             %
 %                                                                             %
@@ -339,6 +398,7 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
 %
 %
 */
+#if defined(POSIX)
 static size_t
 UnixShellTextEscape(char *dst, const char *src, const size_t size)
 {
@@ -382,6 +442,7 @@ UnixShellTextEscape(char *dst, const char *src, const size_t size)
 
   return length;
 }
+#endif /* POSIX */
 
 MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
   const char *decode,const char *encode,ExceptionInfo *exception)
@@ -673,7 +734,6 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
   MagickFreeMemory(commands);
   return(status != False);
 }
-
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -703,14 +763,15 @@ MagickExport unsigned int InvokeDelegate(ImageInfo *image_info,Image *image,
 %      executing it.
 %
 %    o command: The address of a character string containing the command to
-%      execute.
+%      execute.  The command is formulated through direct FormatString()
+%      substitutions rather than using TranslateText.
 %
 %
 */
 MagickExport unsigned int InvokePostscriptDelegate(const unsigned int verbose,
   const char *command)
 {
-#if defined(HasGS) || defined(WIN32)
+#if defined(HasGS) || defined(MSWINDOWS)
   char
     **argv;
 
@@ -719,13 +780,13 @@ MagickExport unsigned int InvokePostscriptDelegate(const unsigned int verbose,
 
   int
     argc,
-    code,
+    pexit_code,
     status;
 
   register long
     i;
 
-#if defined(WIN32)
+#if defined(MSWINDOWS)
   const GhostscriptVectors
     *gs_func;
 
@@ -762,22 +823,49 @@ MagickExport unsigned int InvokePostscriptDelegate(const unsigned int verbose,
       (void) fputs("[ghostscript library]",stdout);
       (void) fputs(strchr(command,' '),stdout);
     }
+  /*
+    Allocate an interpreter.
+  */
+  interpreter = (gs_main_instance *) NULL;
   status=(gs_func->new_instance)(&interpreter,(void *) NULL);
   if (status < 0)
-    return(False);
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "Failed to allocate Ghostscript interpreter!");
+      return(False);
+    }
+  /*
+    Initialize interpreter with argument list.
+  */
   argv=StringToArgv(command,&argc);
   if (argv == (char **) NULL)
-    return(False);
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "Failed to allocate Ghostscript argument list!");
+      return(False);
+    }
   status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
   if (status == 0)
-    status=(gs_func->run_string)
-     (interpreter,"systemdict /start get exec\n",0,&code);
+    {
+      status=(gs_func->run_string)
+        (interpreter,"systemdict /start get exec\n",0,&pexit_code);
+      if ((status == 0) || (status <= -100))
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "Ghostscript returns status %d, exit code %d",
+                              status,pexit_code);
+    }
+  /*
+    Exit interpreter.
+  */
   (gs_func->exit)(interpreter);
+  /*
+    Deallocate interpreter.
+  */
   (gs_func->delete_instance)(interpreter);
   for (i=0; i < argc; i++)
     MagickFreeMemory(argv[i]);
   MagickFreeMemory(argv);
-  if ((status == 0) || (status == -101))
+  if ((status == 0) || (status <= -100))
     return(False);
   return(True);
 #else
@@ -843,7 +931,7 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
       continue;
     *delegate='\0';
     if (p->encode != (char *) NULL)
-      (void) strncpy(delegate,p->encode,MaxTextExtent-1);
+      (void) strlcpy(delegate,p->encode,MaxTextExtent);
     (void) strcat(delegate,"        ");
     delegate[8]='\0';
     commands=StringToList(p->commands);
@@ -871,7 +959,7 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
       for (s=commands[0]; length < command_length; s+=formatted_chars)
         {
           if (s != commands[0])
-            fprintf(file,"%*s",command_start_column,"");
+            (void) fprintf(file,"%*s",command_start_column,"");
           strip_length=screen_width-command_start_column;
           if (length+strip_length < command_length)
             {
@@ -883,7 +971,7 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
             }
           formatted_chars=fprintf(file,"%.*s",strip_length,s);
           length+=formatted_chars;
-          fprintf(file,"\n");
+          (void) fprintf(file,"\n");
           if (formatted_chars <= 0)
             break;
         }
@@ -928,7 +1016,7 @@ MagickExport unsigned int ListDelegateInfo(FILE *file,ExceptionInfo *exception)
 %
 %
 */
-#if defined(WIN32)
+#if defined(MSWINDOWS)
 static void CatDelegatePath(char *path,
                                const char *binpath,
                                const char *command)
@@ -941,7 +1029,7 @@ static void CatDelegatePath(char *path,
   strcpy(path,command);
   return;
 }
-#endif /* defined(WIN32) */
+#endif /* defined(MSWINDOWS) */
 static unsigned int ReadConfigureFile(const char *basename,
   const unsigned long depth,ExceptionInfo *exception)
 {
@@ -974,7 +1062,7 @@ static unsigned int ReadConfigureFile(const char *basename,
       GetToken(q,&q,token);
       if (*token == '\0')
         break;
-      (void) strncpy(keyword,token,MaxTextExtent-1);
+      (void) strlcpy(keyword,token,MaxTextExtent);
       if (LocaleNCompare(keyword,"<!--",4) == 0)
         {
           /*
@@ -991,7 +1079,7 @@ static unsigned int ReadConfigureFile(const char *basename,
           */
           while ((*token != '>') && (*q != '\0'))
             {
-              (void) strncpy(keyword,token,MaxTextExtent-1);
+              (void) strlcpy(keyword,token,MaxTextExtent);
               GetToken(q,&q,token);
               if (*token != '=')
                 continue;
@@ -1007,9 +1095,8 @@ static unsigned int ReadConfigureFile(const char *basename,
 
                       GetPathComponent(path,HeadPath,filename);
                       if (*filename != '\0')
-                        (void) strcat(filename,DirectorySeparator);
-                      (void) strncat(filename,token,MaxTextExtent-
-                                     strlen(filename)-1);
+                        (void) strlcat(filename,DirectorySeparator,MaxTextExtent);
+                      (void) strlcat(filename,token,MaxTextExtent);
                       (void) ReadConfigureFile(filename,depth+1,exception);
                     }
                   if (delegate_list != (DelegateInfo *) NULL)
@@ -1059,7 +1146,7 @@ static unsigned int ReadConfigureFile(const char *basename,
             if (LocaleCompare((char *) keyword,"command") == 0)
               {
                 delegate_list->commands=AllocateString(token);
-#if defined(WIN32)
+#if defined(MSWINDOWS)
                 if (strchr(delegate_list->commands,'@') != (char *) NULL)
                   {
                     char
@@ -1133,7 +1220,7 @@ static unsigned int ReadConfigureFile(const char *basename,
                     SubstituteString((char **) &delegate_list->commands,
                                      "@HPGLDecodeDelegate@",path);
                   }
-#endif /* defined(WIN32) */
+#endif /* defined(MSWINDOWS) */
               } /* LocaleCompare */
             break;
           }
@@ -1177,11 +1264,6 @@ static unsigned int ReadConfigureFile(const char *basename,
         case 'S':
         case 's':
           {
-            if (LocaleCompare((char *) keyword,"spawn") == 0)
-              {
-                delegate_list->spawn=LocaleCompare(token,"True") == 0;
-                break;
-              }
             if (LocaleCompare((char *) keyword,"stealth") == 0)
               {
                 delegate_list->stealth=LocaleCompare(token,"True") == 0;
