@@ -37,11 +37,10 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#if defined(WIN32) || defined(__CYGWIN__)
+#if defined(MSWINDOWS) || defined(__CYGWIN__)
 # include "magick/nt_feature.h"
 #endif
 #include "magick/blob.h"
-#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
@@ -49,6 +48,7 @@
 #include "magick/magic.h"
 #include "magick/magick.h"
 #include "magick/module.h"
+#include "magick/pixel_cache.h"
 #include "magick/registry.h"
 #include "magick/resource.h"
 #include "magick/render.h"
@@ -71,6 +71,9 @@ typedef enum  {
   InitInitialized
 } MagickInitializationState;
 
+static void
+  DestroyMagickInfoList(void);
+
 static SemaphoreInfo
   *magick_semaphore = (SemaphoreInfo *) NULL;
 
@@ -78,7 +81,10 @@ static MagickInfo
   *magick_list = (MagickInfo *) NULL;
 
 static MagickInitializationState MagickInitialized = InitDefault;
+static CoderClass MinimumCoderClass = UnstableCoderClass;
 
+static void DestroyMagickInfo(MagickInfo** magick_info);
+static void DestroyMagickInfoList(void);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -105,28 +111,32 @@ MagickExport void DestroyMagick(void)
     return;
 
 #if defined(HasX11)
-  XDestroyX11Resources();
+  MagickXDestroyX11Resources();
 #endif
-  DestroyColorInfo();
-  DestroyDelegateInfo();
-  DestroyTypeInfo();
-  DestroyMagicInfo();
-  DestroyMagickInfo();
-  DestroyConstitute();
-  DestroyMagickRegistry();
-  DestroyMagickResources();
-  DestroyTemporaryFiles();
-#if defined(WIN32)
-  NTGhostscriptUnLoadDLL();
-#endif /* defined(WIN32) */
+  DestroyColorInfo();           /* Color database */
+  DestroyDelegateInfo();        /* External delegate information */
+  DestroyTypeInfo();            /* Font information */
+  DestroyMagicInfo();           /* File format detection */
+  DestroyMagickInfoList();      /* Coder registrations + modules */
+  DestroyConstitute();          /* Constitute semaphore */
+  DestroyMagickRegistry();      /* Registered images */
+  DestroyMagickResources();     /* Resource semaphore */
+  DestroyTemporaryFiles();      /* Temporary files */
+#if defined(MSWINDOWS)
+  NTGhostscriptUnLoadDLL();     /* Ghostscript DLL */
+#endif /* defined(MSWINDOWS) */
   /*
     Destroy logging last since some components log their destruction.
   */
-  DestroyLogInfo();
-  DestroySemaphore();
+  DestroyLogInfo();             /* Logging configuration */
+  DestroySemaphore();           /* Semaphores framework */
 
   MagickInitialized=InitUninitialized;
+
 }
+/*
+  Destroy MagickInfo structure.
+*/
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -139,14 +149,50 @@ MagickExport void DestroyMagick(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  DestroyMagickInfo() deallocates memory associated MagickInfo list.
+%  DestroyMagickInfo() destroys a MagickInfo structure.
 %
-%  The format of the DestroyMagickInfo method is:
+%  The format of the DestroyMagickInfoList method is:
 %
-%      void DestroyMagickInfo(void)
+%      void DestroyMagickInfo(MagickInfo** magick_info)
 %
 */
-MagickExport void DestroyMagickInfo(void)
+static void DestroyMagickInfo(MagickInfo** magick_info)
+{
+  MagickInfo
+    *p;
+
+  p=*magick_info;
+  if (p)
+    {
+      p->name=0;
+      p->description=0;
+      p->version=0;
+      p->note=0;
+      p->module=0;
+      MagickFreeMemory(p);
+      *magick_info=p;
+    }
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   D e s t r o y M a g i c k I n f o L i s t                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  DestroyMagickInfo() deallocates memory associated with the MagickInfo list.
+%
+%  The format of the DestroyMagickInfoList method is:
+%
+%      void DestroyMagickInfoList(void)
+%
+*/
+static void DestroyMagickInfoList(void)
 {
   MagickInfo
     *magick_info;
@@ -167,19 +213,20 @@ MagickExport void DestroyMagickInfo(void)
     entries anyway.
   */
   AcquireSemaphoreInfo(&magick_semaphore);
+#if defined(BuildMagickModules)
+  if (magick_list != (MagickInfo *) NULL)
+    (void) printf("Warning: module registrations are still present!\n");
+#endif /* defined(BuildMagickModules) */
   for (p=magick_list; p != (MagickInfo *) NULL; )
   {
     magick_info=p;
     p=p->next;
 
-    printf("Warning: module registration for \"%s\" from module \"%s\" still present!\n",
-           magick_info->name, magick_info->module);
-    MagickFreeMemory(magick_info->name);
-    MagickFreeMemory(magick_info->description);
-    MagickFreeMemory(magick_info->version);
-    MagickFreeMemory(magick_info->note);
-    MagickFreeMemory(magick_info->module);
-    MagickFreeMemory(magick_info);
+#if !defined(BuildMagickModules)
+    (void) printf("Warning: module registration for \"%s\" from module \"%s\" still present!\n",
+                  magick_info->name, magick_info->module);
+#endif /* !defined(BuildMagickModules) */
+    DestroyMagickInfo(&magick_info);
   }
   magick_list=(MagickInfo *) NULL;
   LiberateSemaphoreInfo(&magick_semaphore);
@@ -260,92 +307,90 @@ MagickExport const char *GetImageMagick(const unsigned char *magick,
 %
 %
 */
-MagickExport const MagickInfo *GetMagickInfo(const char *name,
-  ExceptionInfo *exception)
+static MagickInfo *GetMagickInfoEntryLocked(const char *name)
 {
   register MagickInfo
     *p;
 
-#if defined(SupportMagickModules)
-  if ((name != (const char *) NULL) && (LocaleCompare(name,"*") == 0))
-    {
-      /*
-        If all modules are requested, then use OpenModules to load
-        all modules.
-      */
-      if (!OpenModules(exception))
-        return 0;
-    }
-#endif /* #if defined(SupportMagickModules) */
-
   AcquireSemaphoreInfo(&magick_semaphore);
-  if (magick_list != (MagickInfo *) NULL)
-    LiberateSemaphoreInfo(&magick_semaphore);
-  else
-    {
-      /*
-        Register image formats.
-      */
-      LiberateSemaphoreInfo(&magick_semaphore);
 
-#if defined(SupportMagickModules)
-      /*
-        Load module aliases and check for any error
-      */
-      if (!GetModuleInfo((char *) NULL,exception))
-        return 0;
-#endif
+  p=magick_list;
+
+  if ((name != (const char *) NULL) && (name[0] != '*'))
+    {
+      for (p=magick_list; p != (MagickInfo *) NULL; p=p->next)
+        if (LocaleCompare(p->name,name) == 0)
+          break;
+      
+      if (p != (MagickInfo *) NULL)
+        if (p != magick_list)
+          {
+            /*
+              Self-adjusting list.
+            */
+            if (p->previous != (MagickInfo *) NULL)
+              p->previous->next=p->next;
+            if (p->next != (MagickInfo *) NULL)
+              p->next->previous=p->previous;
+            p->previous=(MagickInfo *) NULL;
+            p->next=magick_list;
+            magick_list->previous=p;
+            magick_list=p;
+          }
+    }
+
+  LiberateSemaphoreInfo(&magick_semaphore);
+
+  return p;
+}
+MagickExport const MagickInfo *GetMagickInfo(const char *name,
+  ExceptionInfo *ARGUNUSED(exception))
+{
 #if !defined(BuildMagickModules)
+  if ((magick_list == (MagickInfo *) NULL) &&
+      (GetMagickInfoEntryLocked(NULL) == (MagickInfo *) NULL))
+    {
       /*
         Register all static modules
       */
       RegisterStaticModules();
-#endif
     }
-  if ((name == (const char *) NULL) ||  (LocaleCompare(name,"*") == 0))
-    return((const MagickInfo *) magick_list);
+#endif /* !defined(BuildMagickModules) */
+
   /*
-    Find name in list
+    If name is NULL, then return head of list (as it currently
+    exists).
   */
-  AcquireSemaphoreInfo(&magick_semaphore);
-  for (p=magick_list; p != (MagickInfo *) NULL; p=p->next)
-    if (LocaleCompare(p->name,name) == 0)
-      break;
+  if (name == (const char *) NULL)
+    return GetMagickInfoEntryLocked(name);
+
 #if defined(SupportMagickModules)
-  if (p == (MagickInfo *) NULL)
+  /*
+    Load module aliases (does nothing if already loaded)
+  */
+  (void) GetModuleInfo((char *) NULL,exception);
+
+  if (name[0] == '*')
+    {
+      /*
+        If all modules are requested, then use OpenModules to load all
+        modules.
+      */
+      (void) OpenModules(exception);
+    }
+  else
     {
       /*
         Try to load a supporting module.
       */
-      if (*name != '\0')
-        {
-          /* Pass all exceptions up */
-          LiberateSemaphoreInfo(&magick_semaphore);
-          (void) OpenModule(name,exception);
-          AcquireSemaphoreInfo(&magick_semaphore);
-        }
-      for (p=magick_list; p != (MagickInfo *) NULL; p=p->next)
-        if (LocaleCompare(p->name,name) == 0)
-          break;
+      (void) OpenModule(name,exception);
     }
 #endif /* #if defined(SupportMagickModules) */
-  if (p != (MagickInfo *) NULL)
-    if (p != magick_list)
-      {
-        /*
-          Self-adjusting list.
-        */
-        if (p->previous != (MagickInfo *) NULL)
-          p->previous->next=p->next;
-        if (p->next != (MagickInfo *) NULL)
-          p->next->previous=p->previous;
-        p->previous=(MagickInfo *) NULL;
-        p->next=magick_list;
-        magick_list->previous=p;
-        magick_list=p;
-      }
-  LiberateSemaphoreInfo(&magick_semaphore);
-  return((const MagickInfo *) p);
+
+  /*
+    Return whatever we've got
+  */
+  return GetMagickInfoEntryLocked(name);
 }
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -408,7 +453,7 @@ MagickExport MagickInfo **GetMagickInfoArray(ExceptionInfo *exception)
   /*
     Load all modules and obtain pointer to head of list
   */
-  GetMagickInfo("*",exception);
+  (void) GetMagickInfo("*",exception);
   if (!magick_list)
     return 0;
 
@@ -431,7 +476,7 @@ MagickExport MagickInfo **GetMagickInfoArray(ExceptionInfo *exception)
       ThrowException(exception,ResourceLimitError,MemoryAllocationFailed,0);
       return False;
     }
-  memset((void **)array,0,sizeof(MagickInfo *)*(entries+1));
+  (void) memset((void **)array,0,sizeof(MagickInfo *)*(entries+1));
 
   /*
     Add entries to array
@@ -504,7 +549,7 @@ static Sigfunc *MagickSignal(int signo, Sigfunc *func)
     oact;
 
   act.sa_handler=func;
-  sigemptyset(&act.sa_mask);
+  (void) sigemptyset(&act.sa_mask);
   act.sa_flags=0;
 #  if defined(SA_INTERRUPT)  /* SunOS */
   act.sa_flags |= SA_INTERRUPT;
@@ -538,7 +583,7 @@ static Sigfunc *MagickCondSignal(int signo, Sigfunc *func)
         setting.
       */
       if (o_handler != SIG_DFL)
-        MagickSignal(signo,o_handler);
+        (void) MagickSignal(signo,o_handler);
       else
         (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
           "Registered signal handler for signal ID %d",signo);
@@ -563,31 +608,58 @@ static Sigfunc *MagickCondSignal(int signo, Sigfunc *func)
   While support for signal is definitely broken under Windows, the good
   news is that it seems to be unlikely to generate a signal we care about.
 */
-static RETSIGTYPE MagickSignalHandler(int signo)
+static RETSIGTYPE MagickPanicSignalHandler(int signo)
 {
-  /*
-    Release resources
-  */
-  DestroyMagick();
+  /* fprintf(stderr,"Caught panic signal %d\n", signo); */
 
   /*
     Restore default handling for the signal
   */
-  MagickSignal(signo,SIG_DFL);
+  (void) MagickSignal(signo,SIG_DFL);
+
+  /*
+    Release resources
+  */
+  DestroyTemporaryFiles();
 
   /*
     Raise signal again to invoke default handler
     This may cause a core dump or immediate exit.
   */
 #if defined(HAVE_RAISE)
-  raise(signo);
+  (void) fflush(stdout);
+  (void) raise(signo);
 #endif
 
-  Exit(signo);
+  SignalHandlerExit(signo);
+}
+static RETSIGTYPE MagickSignalHandler(int signo)
+{
+  /* fprintf(stderr,"Caught signal %d\n", signo); */
+
+  /*
+    Restore default handling for the signal
+  */
+  (void) MagickSignal(signo,SIG_DFL);
+
+  /*
+    Release resources
+  */
+  DestroyMagick();
+
+  /*
+    Raise signal again to invoke default handler
+    This may cause a core dump or immediate exit.
+  */
+#if defined(HAVE_RAISE)
+  (void) raise(signo);
+#endif
+
+  SignalHandlerExit(signo);
 }
 
 /*
-   The goal of this routine is to determine whether tha passed
+   The goal of this routine is to determine whether the passed
    string is a valid and complete path to a file within the 
    filesystem
  */
@@ -602,7 +674,7 @@ static unsigned int IsValidFilesystemPath(const char *path)
       */
       if ((*path == *DirectorySeparator))
         return IsAccessibleNoLogging(path);
-#elif defined(WIN32)
+#elif defined(MSWINDOWS)
       /* For Windows we check to see if the path passed seems to be a 
          pathof any kind (contains delimiters) or seem to be either UNC
          path or one with a drive letter spec in it: \\Server\share, C:\
@@ -624,7 +696,7 @@ static unsigned int IsValidFilesystemPath(const char *path)
 /*
   Try and figure out the path and name of the client
  */
-MagickExport void InitializeMagickClientPathAndName(const char *path)
+MagickExport void InitializeMagickClientPathAndName(const char *ARGUNUSED(path))
 {
 #if !defined(UseInstalledMagick)
   const char
@@ -661,12 +733,12 @@ MagickExport void InitializeMagickClientPathAndName(const char *path)
       else
         {
           /*
-            The call to the OS failed so we try one last kludgy thing
-            - we call getcwd and sling whatever seems to be a filename
-            passed by the caller onto the end and see if that works
+            The call to the OS failed so we try to compute a usable
+            path based on the current working directory or by
+            searching the executable search path.
           */
           if (path)
-            (void) strncpy(execution_path,path,MaxTextExtent-1);
+            (void) strlcpy(execution_path,path,MaxTextExtent);
           if (GetExecutionPathUsingName(execution_path))
             {
               (void) DefineClientPathAndName(execution_path);
@@ -691,7 +763,7 @@ MagickExport void InitializeMagickClientPathAndName(const char *path)
         This is the easy one. The caller gave us the correct and
         working path to the application, so we just use it
       */
-      (void) strncpy(execution_path,path,MaxTextExtent-1);
+      (void) strlcpy(execution_path,path,MaxTextExtent);
       (void) DefineClientPathAndName(execution_path);
       (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),"Valid path \"%s\"",spath);
     }
@@ -708,20 +780,20 @@ MagickExport void InitializeMagickSignalHandlers(void)
   (void) MagickCondSignal(SIGHUP,MagickSignalHandler);
 #endif
   /* interrupt (rubout), default terminate */
-#if defined(SIGINT) && !defined(WIN32)
+#if defined(SIGINT) && !defined(MSWINDOWS)
   (void) MagickCondSignal(SIGINT,MagickSignalHandler);
 #endif
   /* quit (ASCII FS), default terminate with core */
 #if defined(SIGQUIT)
-  (void) MagickCondSignal(SIGQUIT,MagickSignalHandler);
+  (void) MagickCondSignal(SIGQUIT,MagickPanicSignalHandler);
 #endif
   /* software-triggered abort, default terminate with core */
 #if defined(SIGABRT)
-  (void) MagickCondSignal(SIGABRT,MagickSignalHandler);
+  (void) MagickCondSignal(SIGABRT,MagickPanicSignalHandler);
 #endif
   /* floating point exception, default terminate with core */
 #if defined(SIGFPE)
-  (void) MagickCondSignal(SIGFPE,MagickSignalHandler);
+  (void) MagickCondSignal(SIGFPE,MagickPanicSignalHandler);
 #endif
   /* software termination signal from kill, default terminate */
 #if defined(SIGTERM)
@@ -729,60 +801,62 @@ MagickExport void InitializeMagickSignalHandlers(void)
 #endif
   /* exceeded cpu limit, default terminate with core */
 #if defined(SIGXCPU)
-  (void) MagickCondSignal(SIGXCPU,MagickSignalHandler);
+  (void) MagickCondSignal(SIGXCPU,MagickPanicSignalHandler);
 #endif
   /* exceeded file size limit, default terminate with core */
 #if defined(SIGXFSZ)
-  (void) MagickCondSignal(SIGXFSZ,MagickSignalHandler);
+  (void) MagickCondSignal(SIGXFSZ,MagickPanicSignalHandler);
 #endif
 }
 
 MagickExport void InitializeMagick(const char *path)
 {
+  const char
+    *p;
+
   /* NOTE: This routine sets up the path to the client which needs to
      be determined before almost anything else works right. This also
      includes logging!!! So we can't start logging until the path is
      actually saved. As soon as we know what the path is we make the
      same call to DefineClientSettings to set it up. Please make sure
      that this rule is followed in any future updates the this code!!!
-   */
-
+  */
   if (MagickInitialized == InitInitialized)
     return;
   SPINLOCK_WAIT;
   MagickInitialized=InitInitialized;
   SPINLOCK_RELEASE;
-
-#if defined(WIN32)
+  
+#if defined(MSWINDOWS)
 # if defined(_DEBUG) && !defined(__BORLANDC__)
   {
     int
       debug;
-
+    
     debug=_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
     debug|=_CRTDBG_CHECK_ALWAYS_DF | _CRTDBG_DELAY_FREE_MEM_DF |
-            _CRTDBG_LEAK_CHECK_DF;
+      _CRTDBG_LEAK_CHECK_DF;
     // debug=_CrtSetDbgFlag(debug);
     // _ASSERTE(_CrtCheckMemory());
   }
 # endif /* defined(_DEBUG) */
-#endif /* defined(WIN32) */
-
+#endif /* defined(MSWINDOWS) */
+  
   (void) setlocale(LC_ALL,"");
   (void) setlocale(LC_NUMERIC,"C");
-
+  
   /* Seed the random number generator */
   srand(time(0));
-
+  
   /* Initialize semaphores */
   InitializeSemaphore();
-
+  
   /*
     Set logging flags using the value of MAGICK_DEBUG if it is set in
     the environment.
   */
-  if (getenv("MAGICK_DEBUG"))
-    (void) SetLogEventMask(getenv("MAGICK_DEBUG"));
+  if ((p=getenv("MAGICK_DEBUG")) != (const char *) NULL)
+    (void) SetLogEventMask(p);
 
   /*
     Establish the path, filename, and display name of the client app
@@ -799,7 +873,8 @@ MagickExport void InitializeMagick(const char *path)
     Set logging flags using the value of MAGICK_DEBUG if it is set in
     the environment.
   */
-  (void) SetLogEventMask(getenv("MAGICK_DEBUG"));
+  if ((p=getenv("MAGICK_DEBUG")) != (const char *) NULL)
+    (void) SetLogEventMask(p);
 
   /*
     Establish signal handlers for common signals
@@ -814,6 +889,19 @@ MagickExport void InitializeMagick(const char *path)
      to the client is setup */
   InitializeMagickResources();
 
+  /*
+    Adjust minimum coder class if requested.
+  */
+  if ((p=getenv("MAGICK_CODER_STABILITY")) != (const char *) NULL)
+    {
+      if (LocaleCompare(p,"UNSTABLE") == 0)
+        MinimumCoderClass=UnstableCoderClass;
+      else if (LocaleCompare(p,"STABLE") == 0)
+        MinimumCoderClass=StableCoderClass;
+      else if (LocaleCompare(p,"PRIMARY") == 0)
+        MinimumCoderClass=PrimaryCoderClass;
+    }
+
   /*
     Initialize module loader
   */
@@ -843,7 +931,7 @@ MagickExport void InitializeMagick(const char *path)
 %
 %  The format of the IsMagickConflict method is:
 %
-%      unsigned int IsMagickConflict(const char *magick)
+%      MagickBool IsMagickConflict(const char *magick)
 %
 %  A description of each parameter follows:
 %
@@ -854,19 +942,13 @@ MagickExport void InitializeMagick(const char *path)
 %
 %
 */
-MagickExport int unsigned IsMagickConflict(const char *magick)
+MagickExport MagickBool IsMagickConflict(const char *magick)
 {
   assert(magick != (char *) NULL);
-#if defined(macintosh)
-  return(MACIsMagickConflict(magick));
-#endif
-#if defined(vms)
-  return(VMSIsMagickConflict(magick));
-#endif
-#if defined(WIN32) || defined(__CYGWIN__)
+#if defined(MSWINDOWS) || defined(__CYGWIN__)
   return(NTIsMagickConflict(magick));
 #endif
-  return(False);
+  return(MagickFalse);
 }
 
 /*
@@ -884,7 +966,7 @@ MagickExport int unsigned IsMagickConflict(const char *magick)
 %
 %  The format of the ListMagickInfo method is:
 %
-%      unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
+%      MagickPassFail ListMagickInfo(FILE *file,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -894,7 +976,7 @@ MagickExport int unsigned IsMagickConflict(const char *magick)
 %
 %
 */
-MagickExport unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
+MagickExport MagickPassFail ListMagickInfo(FILE *file,ExceptionInfo *exception)
 {
 
   MagickInfo
@@ -910,16 +992,21 @@ MagickExport unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
   if (!magick_array)
     return False;
 
-  (void) fprintf(file,"   Format  Mode  Description\n");
+  (void) fprintf(file,"   Format L  Mode  Description\n");
   (void) fprintf(file,"--------------------------------------------------------"
-    "-----------------------\n");
+    "------------------------\n");
   for (i=0; magick_array[i] != 0; i++)
   {
     if (magick_array[i]->stealth)
       continue;
-    (void) fprintf(file,"%9s%c  %c%c%c",magick_array[i]->name ? magick_array[i]->name : "",
-      magick_array[i]->blob_support ? '*' : ' ',magick_array[i]->decoder ? 'r' : '-',
-      magick_array[i]->encoder ? 'w' : '-',magick_array[i]->encoder && magick_array[i]->adjoin ? '+' : '-');
+    (void) fprintf(file,"%9s %c  %c%c%c",
+                   magick_array[i]->name ? magick_array[i]->name : "",
+                   (magick_array[i]->coder_class == PrimaryCoderClass ? 'P' :
+                    (magick_array[i]->coder_class == StableCoderClass ? 'S' :
+                     'U')),
+                   (magick_array[i]->decoder ? 'r' : '-'),
+                   (magick_array[i]->encoder ? 'w' : '-'),
+                   ((magick_array[i]->encoder && magick_array[i]->adjoin) ? '+' : '-'));
     if (magick_array[i]->description != (char *) NULL)
       (void) fprintf(file,"  %.1024s",magick_array[i]->description);
     if (magick_array[i]->version != (char *) NULL)
@@ -945,10 +1032,10 @@ MagickExport unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
           }
       }
   }
-  (void) fprintf(file,"\n* native blob support\n\n");
+  (void) fprintf(file,"\n Meaning of 'L': P=Primary, S=Stable, U=Unstable\n");
   (void) fflush(file);
   MagickFreeMemory(magick_array);
-  return(True);
+  return(MagickPass);
 }
 
 /*
@@ -967,7 +1054,7 @@ MagickExport unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
 %
 %  The format of the ListModuleMap method is:
 %
-%      unsigned int ListModuleMap(FILE *file,ExceptionInfo *exception)
+%      MagickPassFail ListModuleMap(FILE *file,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -977,7 +1064,7 @@ MagickExport unsigned int ListMagickInfo(FILE *file,ExceptionInfo *exception)
 %
 %
 */
-MagickExport unsigned int ListModuleMap(FILE *file,ExceptionInfo *exception)
+MagickExport MagickPassFail ListModuleMap(FILE *file,ExceptionInfo *exception)
 {
   MagickInfo
     **magick_array;
@@ -990,7 +1077,7 @@ MagickExport unsigned int ListModuleMap(FILE *file,ExceptionInfo *exception)
 
    magick_array=GetMagickInfoArray(exception);
    if ((!magick_array) || (exception->severity > UndefinedException))
-     return False;
+     return MagickFail;
 
    (void) fprintf(file, "<?xml version=\"1.0\"?>\n");
    (void) fprintf(file, "<!-- %s -->\n",GetMagickCopyright());
@@ -1013,7 +1100,7 @@ MagickExport unsigned int ListModuleMap(FILE *file,ExceptionInfo *exception)
 
    MagickFreeMemory(magick_array);
 
-   return(True);
+   return(MagickPass);
 }
 
 /*
@@ -1126,57 +1213,36 @@ MagickExport char *MagickToMime(const char *magick)
 */
 MagickExport MagickInfo *RegisterMagickInfo(MagickInfo *magick_info)
 {
-  register MagickInfo
-    *p;
-
-  /*
-    Delete any existing name.
-  */
   assert(magick_info != (MagickInfo *) NULL);
   assert(magick_info->signature == MagickSignature);
+
+  /*
+    Remove any existing entry.
+  */
   (void) UnregisterMagickInfo(magick_info->name);
-  AcquireSemaphoreInfo(&magick_semaphore);
-  magick_info->previous=(MagickInfo *) NULL;
-  magick_info->next=(MagickInfo *) NULL;
-  if (magick_list == (MagickInfo *) NULL)
+
+  /*
+    Verify that coder stability level is sufficient.
+  */
+  if (magick_info->coder_class >= MinimumCoderClass)
     {
       /*
-        Start magick list.
+        Add to front of list.
       */
+      AcquireSemaphoreInfo(&magick_semaphore);
+      magick_info->previous=(MagickInfo *) NULL;
+      magick_info->next=magick_list;
+      if (magick_info->next != (MagickInfo *) NULL)
+        magick_info->next->previous=magick_info;
       magick_list=magick_info;
       LiberateSemaphoreInfo(&magick_semaphore);
       return(magick_info);
     }
+
   /*
-    Tag is added in lexographic order.
+    Discard registration and return NULL.
   */
-  for (p=magick_list; p->next != (MagickInfo *) NULL; p=p->next)
-    if (LocaleCompare(p->name,magick_info->name) >= 0)
-      break;
-  if (LocaleCompare(p->name,magick_info->name) < 0)
-    {
-      /*
-        Add entry after target.
-      */
-      magick_info->next=p->next;
-      p->next=magick_info;
-      magick_info->previous=p;
-      if (magick_info->next != (MagickInfo *) NULL)
-        magick_info->next->previous=magick_info;
-      LiberateSemaphoreInfo(&magick_semaphore);
-      return(magick_info);
-    }
-  /*
-    Add entry before target.
-  */
-  magick_info->next=p;
-  magick_info->previous=p->previous;
-  p->previous=magick_info;
-  if (magick_info->previous != (MagickInfo *) NULL)
-    magick_info->previous->next=magick_info;
-  if (p == magick_list)
-    magick_list=magick_info;
-  LiberateSemaphoreInfo(&magick_semaphore);
+  DestroyMagickInfo(&magick_info);
   return(magick_info);
 }
 
@@ -1219,10 +1285,15 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
     MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
       UnableToAllocateMagickInfo);
   (void) memset(magick_info,0,sizeof(MagickInfo));
-  magick_info->name=AcquireString(name);
-  magick_info->adjoin=True;
-  magick_info->blob_support=True;
-  magick_info->thread_support=True;
+  magick_info->name=name;
+  magick_info->adjoin=MagickTrue;
+  magick_info->raw=MagickFalse;
+  magick_info->stealth=MagickFalse;
+  magick_info->seekable_stream=MagickFalse;
+  magick_info->blob_support=MagickTrue;
+  magick_info->thread_support=MagickTrue;
+  magick_info->coder_class=StableCoderClass;
+  magick_info->extension_treatment=HintExtensionTreatment;
   magick_info->signature=MagickSignature;
   return(magick_info);
 }
@@ -1239,7 +1310,8 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  Method UnregisterMagickInfo removes a name from the magick info list.  It
-%  returns False if the name does not exist in the list otherwise True.
+%  returns MagickFail if the name does not exist in the list otherwise
+%  MagickPass.
 %
 %  The format of the UnregisterMagickInfo method is:
 %
@@ -1254,7 +1326,7 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
 %      looking for.
 %
 */
-MagickExport unsigned int UnregisterMagickInfo(const char *name)
+MagickExport MagickPassFail UnregisterMagickInfo(const char *name)
 {
   MagickInfo
     *magick_info;
@@ -1266,7 +1338,7 @@ MagickExport unsigned int UnregisterMagickInfo(const char *name)
     status;
 
   assert(name != (const char *) NULL);
-  status=False;
+  status=MagickFail;
   AcquireSemaphoreInfo(&magick_semaphore);
   for (p=magick_list; p != (MagickInfo *) NULL; p=p->next)
   {
@@ -1279,17 +1351,8 @@ MagickExport unsigned int UnregisterMagickInfo(const char *name)
     else
       magick_list=p->next;
     magick_info=p;
-    MagickFreeMemory(magick_info->name);
-    if (magick_info->description != (char *) NULL)
-      MagickFreeMemory(magick_info->description);
-    if (magick_info->version != (char *) NULL)
-      MagickFreeMemory(magick_info->version);
-    if (magick_info->note != (char *) NULL)
-      MagickFreeMemory(magick_info->note);
-    if (magick_info->module != (char *) NULL)
-      MagickFreeMemory(magick_info->module);
-    MagickFreeMemory(magick_info);
-    status=True;
+    DestroyMagickInfo(&magick_info);
+    status=MagickPass;
     break;
   }
   LiberateSemaphoreInfo(&magick_semaphore);
