@@ -37,7 +37,7 @@
 */
 #include "magick/studio.h"
 #if defined(SupportMagickModules)
-#if defined(WIN32) || defined(__CYGWIN__)
+#if defined(MSWINDOWS) || defined(__CYGWIN__)
 # include "magick/nt_feature.h"
 #endif
 #include "magick/blob.h"
@@ -161,7 +161,7 @@ static CoderInfo
   *RegisterModule(CoderInfo *,ExceptionInfo *),
   *SetCoderInfo(const char *);
 
-#if FUNCTION_UNUSED
+#if 0
 static const CoderInfo
   *GetCoderInfo(const char *,ExceptionInfo *);
 #endif
@@ -268,12 +268,9 @@ MagickExport void DestroyModuleInfo(void)
   {
     module_info=q;
     q=q->next;
-    if (module_info->path != (char *) NULL)
-      MagickFreeMemory(module_info->path);
-    if (module_info->magick != (char *) NULL)
-      MagickFreeMemory(module_info->magick);
-    if (module_info->name != (char *) NULL)
-      MagickFreeMemory(module_info->name);
+    MagickFreeMemory(module_info->path);
+    MagickFreeMemory(module_info->magick);
+    MagickFreeMemory(module_info->name);
     MagickFreeMemory(module_info);
   }
   module_list=(ModuleInfo *) NULL;
@@ -384,14 +381,14 @@ MagickExport MagickPassFail ExecuteModuleProcess(const char *tag,Image **image,
     method=(unsigned int (*)(Image **,const int,char **))
       lt_dlsym(handle,method_name);
 
-    LogMagickEvent(CoderEvent,GetMagickModule(),
-      "Invoking \"%.1024s\" filter module",tag);
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Invoking \"%.1024s\" filter module",tag);
 
     /* Execute module method */
     if (method != (unsigned int (*)(Image **,const int,char **)) NULL)
       status=(*method)(image,argc,argv);
 
-    LogMagickEvent(CoderEvent,GetMagickModule(),
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
       "Returned from \"%.1024s\" filter module",tag);
 
   }
@@ -432,14 +429,14 @@ MagickExport MagickPassFail ExecuteModuleProcess(const char *tag,Image **image,
 %
 %
 */
-#if FUNCTION_UNUSED
+#if 0
 static const CoderInfo *GetCoderInfo(const char *tag,
   ExceptionInfo *exception)
 {
   register CoderInfo
     *p;
 
-  (void) GetMagicInfo((unsigned char *) NULL,0,exception);
+  (void) InitializeMagicInfo(exception);
   (void) GetModuleInfo(tag,exception);
   if ((tag == (const char *) NULL) || (LocaleCompare(tag,"*") == 0))
     return((const CoderInfo *) coder_list);
@@ -522,7 +519,7 @@ static MagickPassFail FindMagickModule(const char *filename,
   assert(filename != (const char *) NULL);
   assert(path != (char *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
-  (void) strncpy(path,filename,MaxTextExtent-1);
+  (void) strlcpy(path,filename,MaxTextExtent);
   
   if (InitializeModuleSearchPath(module_type,exception) == MagickFail)
     return (status);
@@ -561,9 +558,9 @@ static MagickPassFail FindMagickModule(const char *filename,
       while(MagickMapIterateNext(path_map_iterator,&key))
         {
           if (search_path)
-            ConcatenateString(&search_path,list_seperator);
-          ConcatenateString(&search_path,
-                            MagickMapDereferenceIterator(path_map_iterator,0));
+            (void) ConcatenateString(&search_path,list_seperator);
+          (void) ConcatenateString(&search_path,
+                                   MagickMapDereferenceIterator(path_map_iterator,0));
         }
       
       (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
@@ -663,7 +660,7 @@ static MagickPassFail GetModuleListForDirectory
     if (i >= (long) *max_entries)
       {
         *max_entries<<=1;
-        MagickReallocMemory(list,*max_entries*sizeof(char *));
+        MagickReallocMemory(char **,list,*max_entries*sizeof(char *));
         if (list == (char **) NULL)
           break;
       }
@@ -784,11 +781,11 @@ MagickExport void InitializeMagickModules(void)
               UnableToInitializeModuleLoader,lt_dlerror());
           ltdl_initialized=True;
         }
-      ReadModuleConfigureFile(ModuleFilename,0,&exception);
+      (void) ReadModuleConfigureFile(ModuleFilename,0,&exception);
     }
   LiberateSemaphoreInfo(&module_semaphore);
-  InitializeModuleSearchPath(MagickCoderModule,&exception);
-  InitializeModuleSearchPath(MagickFilterModule,&exception);
+  (void) InitializeModuleSearchPath(MagickCoderModule,&exception);
+  (void) InitializeModuleSearchPath(MagickFilterModule,&exception);
 
   DestroyExceptionInfo(&exception);
 }
@@ -826,7 +823,7 @@ static void AddModulePath(MagickMap path_map, unsigned int *path_index,
     key[MaxTextExtent];
 
   FormatString(key,"%u",*path_index);
-  MagickMapAddEntry(path_map,key,(void *)path,0,exception);
+  (void) MagickMapAddEntry(path_map,key,(void *)path,0,exception);
   (*path_index)++;
 }
 
@@ -939,10 +936,10 @@ MagickPassFail InitializeModuleSearchPath(MagickModuleType module_type,
             length=end-start;
           if (length > MaxTextExtent-1)
             length = MaxTextExtent-1;
-          strncpy(buffer,start,length);
+          (void) strncpy(buffer,start,length);
           buffer[length]='\0';
           if (buffer[length-1] != DirectorySeparator[0])
-            strcat(buffer,DirectorySeparator);
+            (void) strcat(buffer,DirectorySeparator);
           AddModulePath(path_map,&path_index,buffer,exception);
           start += length+1;
         }
@@ -973,7 +970,7 @@ MagickPassFail InitializeModuleSearchPath(MagickModuleType module_type,
     return (status);
   }
 # else
-#  if defined(WIN32)
+#  if defined(MSWINDOWS)
   {
     /*
       Locate path via registry key.
@@ -1005,10 +1002,10 @@ MagickPassFail InitializeModuleSearchPath(MagickModuleType module_type,
     LiberateSemaphoreInfo(&module_semaphore);
     return (status);
   }
-#  endif /* defined(WIN32) */
+#  endif /* defined(MSWINDOWS) */
 # endif /* !defined(MagickCoderModulesPath) */
-# if !defined(MagickCoderModulesPath) && !defined(WIN32)
-#  error MagickCoderModulesPath or WIN32 must be defined when UseInstalledMagick is defined
+# if !defined(MagickCoderModulesPath) && !defined(MSWINDOWS)
+#  error MagickCoderModulesPath or MSWINDOWS must be defined when UseInstalledMagick is defined
 # endif
 #else /* end defined(UseInstalledMagick) */
   if (getenv("MAGICK_HOME") != (char *) NULL)
@@ -1059,7 +1056,7 @@ MagickPassFail InitializeModuleSearchPath(MagickModuleType module_type,
           break;
         }
 
-      (void) strncpy(prefix,SetClientPath((char *) NULL),MaxTextExtent-1);
+      (void) strlcpy(prefix,SetClientPath((char *) NULL),MaxTextExtent);
       ChopPathComponents(prefix,1);
       FormatString(path,"%.512s/lib/%s/modules-Q%d/%s/",prefix,
         MagickLibSubdir,QuantumDepth,module_subdir);
@@ -1102,7 +1099,7 @@ MagickPassFail InitializeModuleSearchPath(MagickModuleType module_type,
 %
 %  GetModuleInfo() returns a pointer to a ModuleInfo structure that matches
 %  the specified tag.  If tag is NULL, the head of the module alias list is
-%  returned. If no modules magick are loaded, or the requested alias is not
+%  returned. If no modules are loaded, or the requested alias is not
 %  found, NULL is returned.
 %
 %  The format of the GetModuleInfo method is:
@@ -1230,16 +1227,16 @@ MagickExport unsigned int ListModuleInfo(FILE *file,ExceptionInfo *exception)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  OpenModule() loads a module, and invokes its registration method.  It
-%  returns True on success, and False if there is an error.
+%  returns MagickPass on success, and MagickFail if there is an error.
 %
 %  The format of the OpenModule method is:
 %
-%      unsigned int OpenModule(const char *module,ExceptionInfo *exception)
+%      MagickPassFail OpenModule(const char *module,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method OpenModule returns True if the specified module is
-%      loaded, otherwise False.
+%    o status: Method OpenModule returns MagickPass if the specified module is
+%      loaded, otherwise MagickFail.
 %
 %    o module: a character string that indicates the module to load.
 %
@@ -1247,7 +1244,7 @@ MagickExport unsigned int ListModuleInfo(FILE *file,ExceptionInfo *exception)
 %
 %
 */
-MagickExport unsigned int OpenModule(const char *module,
+MagickExport MagickPassFail OpenModule(const char *module,
   ExceptionInfo *exception)
 {
   {
@@ -1271,14 +1268,25 @@ MagickExport unsigned int OpenModule(const char *module,
       Assign module name from alias.
     */
     assert(module != (const char *) NULL);
-    (void) strncpy(module_name,module,MaxTextExtent-1);
+    (void) strlcpy(module_name,module,MaxTextExtent);
     if (module_list != (ModuleInfo *) NULL)
       for (p=module_list; p != (ModuleInfo *) NULL; p=p->next)
         if (LocaleCompare(p->magick,module) == 0)
           {
-            (void) strncpy(module_name,p->name,MaxTextExtent-1);
+            (void) strlcpy(module_name,p->name,MaxTextExtent);
             break;
           }
+
+    /*
+      Ignore already loaded modules.
+    */
+    for (coder_info=coder_list; coder_info != (CoderInfo *) NULL;
+         coder_info=coder_info->next)
+      if (LocaleCompare(coder_info->tag,module_name) == 0)
+        break;
+    if (coder_info != (CoderInfo *) NULL)
+      return MagickPass;
+
     /*
       Find module file.
     */
@@ -1301,8 +1309,8 @@ MagickExport unsigned int OpenModule(const char *module,
     if ((module_list != (ModuleInfo *) NULL) &&
         (module_list->path != (char *) NULL))
       GetPathComponent(module_list->path,HeadPath,path);
-    (void) strcat(path,DirectorySeparator);
-    (void) strncat(path,module_file,MaxTextExtent-strlen(path)-1);
+    (void) strlcat(path,DirectorySeparator,MaxTextExtent);
+    (void) strlcat(path,module_file,MaxTextExtent);
 #endif
     /*
       Load module
@@ -1315,7 +1323,7 @@ MagickExport unsigned int OpenModule(const char *module,
       {
         FormatString(message,"\"%.1024s: %.1024s\"",path,lt_dlerror());
         ThrowException(exception,ModuleError,UnableToLoadModule,message);
-        return(False);
+        return(MagickFail);
       }
     /*
       Add module to coder module list.
@@ -1324,7 +1332,7 @@ MagickExport unsigned int OpenModule(const char *module,
     if (coder_info == (CoderInfo*) NULL)
       {
         (void) lt_dlclose(handle);
-        return(False);
+        return(MagickFail);
       }
     coder_info->handle=handle;
     (void) time(&coder_info->load_time);
@@ -1340,7 +1348,7 @@ MagickExport unsigned int OpenModule(const char *module,
         FormatString(message,"\"%.1024s: %.1024s\"",module_name,lt_dlerror());
         ThrowException(exception,ModuleError,UnableToRegisterImageFormat,
           message);
-        return(False);
+        return(MagickFail);
       }
 
     (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
@@ -1357,7 +1365,7 @@ MagickExport unsigned int OpenModule(const char *module,
         FormatString(message,"\"%.1024s: %.1024s\"",module_name,lt_dlerror());
         ThrowException(exception,ModuleError,UnableToRegisterImageFormat,
           message);
-        return(False);
+        return(MagickFail);
       }
 
     (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
@@ -1369,7 +1377,7 @@ MagickExport unsigned int OpenModule(const char *module,
     */
     coder_info->register_function();
   }
-  return(True);
+  return(MagickPass);
 }
 
 /*
@@ -1387,7 +1395,7 @@ MagickExport unsigned int OpenModule(const char *module,
 %
 %  The format of the OpenModules method is:
 %
-%      unsigned int OpenModules(ExceptionInfo *exception)
+%      MagickPassFail OpenModules(ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -1397,7 +1405,7 @@ MagickExport unsigned int OpenModule(const char *module,
 %    o exception: Return any errors or warnings in this structure.
 %
 */
-MagickExport unsigned int OpenModules(ExceptionInfo *exception)
+MagickExport MagickPassFail OpenModules(ExceptionInfo *exception)
 {
   (void) GetMagickInfo((char *) NULL,exception);
   {
@@ -1498,7 +1506,7 @@ static unsigned int ReadModuleConfigureFile(const char *basename,
     GetToken(q,&q,token);
     if (*token == '\0')
       break;
-    (void) strncpy(keyword,token,MaxTextExtent-1);
+    (void) strlcpy(keyword,token,MaxTextExtent);
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
@@ -1515,7 +1523,7 @@ static unsigned int ReadModuleConfigureFile(const char *basename,
         */
         while ((*token != '>') && (*q != '\0'))
         {
-          strncpy(keyword,token,MaxTextExtent-1);
+          (void) strlcpy(keyword,token,MaxTextExtent);
           GetToken(q,&q,token);
           if (*token != '=')
             continue;
@@ -1531,9 +1539,8 @@ static unsigned int ReadModuleConfigureFile(const char *basename,
 
                   GetPathComponent(path,HeadPath,filename);
                   if (*filename != '\0')
-                    (void) strcat(filename,DirectorySeparator);
-                  (void) strncat(filename,token,MaxTextExtent-
-                    strlen(filename)-1);
+                    (void) strlcat(filename,DirectorySeparator,MaxTextExtent);
+                  (void) strlcat(filename,token,MaxTextExtent);
                   (void) ReadModuleConfigureFile(filename,depth+1,exception);
                 }
               if (module_list != (ModuleInfo *) NULL)
@@ -1789,9 +1796,9 @@ static void TagToCoderModuleName(const char *tag,char *module_name)
   (void) FormatString(module_name,"%.1024s.la",tag);
   (void) LocaleLower(module_name);
 #else
-#if defined(WIN32)
+#if defined(MSWINDOWS)
   if (LocaleNCompare("IM_MOD_",tag,7) == 0)
-    (void) strncpy(module_name,tag,MaxTextExtent-1);
+    (void) strlcpy(module_name,tag,MaxTextExtent);
   else
     {
 #  if defined(_DEBUG)
@@ -1800,7 +1807,7 @@ static void TagToCoderModuleName(const char *tag,char *module_name)
       FormatString(module_name,"IM_MOD_RL_%.1024s_.dll",tag);
 #  endif /* defined(_DEBUG) */
     }
-#endif  /* defined(WIN32) */
+#endif  /* defined(MSWINDOWS) */
 #endif /* defined(HasLTDL) */
 }
 
@@ -1878,13 +1885,14 @@ static void TagToFunctionName(const char *tag,const char *format,char *function)
   assert(tag != (const char *) NULL);
   assert(format != (const char *) NULL);
   assert(function != (char *) NULL);
-  strncpy(function_name,tag,MaxTextExtent-1);
+  (void) strlcpy(function_name,tag,MaxTextExtent);
   LocaleUpper(function_name);
 
 #if defined(PREFIX_MAGICK_SYMBOLS)
-  snprintf(extended_format,MaxTextExtent-1,"Gm%.1024s",format);
+  (void) strlcpy(extended_format,"Gm",MaxTextExtent);
+  (void) strlcat(extended_format,format,MaxTextExtent);
 #else
-  strncpy(extended_format,format,MaxTextExtent-1);
+  (void) strlcpy(extended_format,format,MaxTextExtent);
 #endif
   FormatString(function,extended_format,function_name);
 }
@@ -1930,19 +1938,25 @@ static unsigned int UnloadModule(const CoderInfo *coder_info,
   assert(coder_info->unregister_function != (void (*)(void)) NULL);
   assert(exception != (ExceptionInfo *) NULL);
 
+  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                        "Unloading \"%s\" module ...", coder_info->tag);
+
   /*
     Invoke module unregister (UnregisterFORMATImage) function
   */
   coder_info->unregister_function();
 
   /*
-    Close module
+    Close module.  Don't close JP2 module since it uses atexit().
   */
-  if(lt_dlclose((ModuleHandle) coder_info->handle))
+  if ( strcmp("JP2",coder_info->tag) != 0 )
     {
-      FormatString(message,"\"%.1024s: %.1024s\"",name,lt_dlerror());
-      ThrowException(exception,ModuleError,FailedToCloseModule,message);
-      status=False;
+      if (lt_dlclose((ModuleHandle) coder_info->handle))
+        {
+          FormatString(message,"\"%.1024s: %.1024s\"",name,lt_dlerror());
+          ThrowException(exception,ModuleError,FailedToCloseModule,message);
+          status=False;
+        }
     }
   return (status);
 }
